from rdstar.branching_fractions import b_meson_bf_calculation as bf_calc
from rdstar.branching_fractions.belle_neutral_b_meson_decay_info import (
    sum_of_belle_b0_to_dss_l_nu_bfs,
    sum_of_belle_removed_b0_bfs,
    belle_bf_b0_to_d_l_nu,
    belle_bf_b0_to_d_tau_nu,
    belle_bf_b0_to_d_st_l_nu,
    belle_bf_b0_to_d_st_tau_nu,
)

__all__ = [
    "MixedBBFComparison",
]


class MixedBBFComparison:

    sum_our_b0_to_d_l_nu_bfs = (
        2 * bf_calc.bf_b0_to_d_l_nu_avg.nominal_value + bf_calc.bf_b0_to_d_tau_nu_avg.nominal_value
    )  # type: float

    sum_our_b0_to_d_st_l_nu_bfs = (
        2 * bf_calc.bf_b0_to_d_st_l_nu_avg.nominal_value + bf_calc.bf_b0_to_d_st_tau_nu_avg.nominal_value
    )  # type: float

    sum_our_b0_to_dss_l_nu_bfs = (
        2 * bf_calc.bf_b0_to_d0st_l_nu_our_cal.nominal_value
        + 2 * bf_calc.bf_b0_to_d_1_l_nu_our_cal.nominal_value
        + 2 * bf_calc.bf_b0_to_dprime1_l_nu_our_cal.nominal_value
        + 2 * bf_calc.bf_b0_to_dst_2_l_nu_our_cal.nominal_value
        + bf_calc.bf_b0_to_d0st_tau_nu_cal.nominal_value
        + bf_calc.bf_b0_to_d_1_tau_nu_cal.nominal_value
        + bf_calc.bf_b0_to_dprime1_tau_nu_cal.nominal_value
        + bf_calc.bf_b0_to_dst_2_tau_nu_cal.nominal_value
    )  # type: float

    sum_our_b0_to_gap_bfs = (
        2 * bf_calc.bf_b0_to_d_1_l_nu_with_d_1_to_d_pi_pi_gap.nominal_value
        + 2 * bf_calc.bf_b0_to_d_eta_l_nu_gap.nominal_value
        + 2 * bf_calc.bf_b0_to_d_st_eta_l_nu_gap.nominal_value
        + 2 * bf_calc.bf_b0_to_d_pi_pi_l_nu_our_gap.nominal_value
        + 2 * bf_calc.bf_b0_to_d_st_pi_pi_l_nu_our_gap.nominal_value
    )

    def get_bf_diff_b0_to_d_l_nu(self) -> float:
        return (2 * belle_bf_b0_to_d_l_nu + belle_bf_b0_to_d_tau_nu) - self.sum_our_b0_to_d_l_nu_bfs

    def get_bf_diff_b0_to_d_st_l_nu(self) -> float:
        return (2 * belle_bf_b0_to_d_st_l_nu + belle_bf_b0_to_d_st_tau_nu) - self.sum_our_b0_to_d_st_l_nu_bfs

    def get_bf_diff_b0_to_dss_l_nu(self) -> float:
        return sum_of_belle_b0_to_dss_l_nu_bfs - self.sum_our_b0_to_dss_l_nu_bfs

    def get_bf_diff_b0_to_gap(self) -> float:
        return 2 * sum_of_belle_removed_b0_bfs - self.sum_our_b0_to_gap_bfs

    # BFs which are not simulated, but considered in the calculation of the updated bfs
    @staticmethod
    def get_missing_b0_bf_sum() -> float:
        return (
            2 * bf_calc.bf_b0_to_dst_2_l_nu_with_dst_2_to_dst_pi_pi_missing.nominal_value
            + 2 * bf_calc.bf_b0_to_d_s_kaon_l_nu_missing.nominal_value
            + 2 * bf_calc.bf_b0_to_d_st_s_kaon_l_nu_missing.nominal_value
            + bf_calc.bf_b0_to_d_1_tau_nu_with_d_1_to_d_pi_pi_gap_missing.nominal_value
            + bf_calc.bf_b0_to_d_eta_tau_nu_gap_missing.nominal_value
            + bf_calc.bf_b0_to_d_st_eta_tau_nu_gap_missing.nominal_value
            + bf_calc.bf_b0_to_d_pi_pi_tau_nu_our_gap_missing.nominal_value
            + bf_calc.bf_b0_to_d_st_pi_pi_tau_nu_our_gap_missing.nominal_value
        )

    def get_total_b0_bf_diff(self) -> float:
        return (
            self.get_bf_diff_b0_to_d_l_nu()
            + self.get_bf_diff_b0_to_d_st_l_nu()
            + self.get_bf_diff_b0_to_dss_l_nu()
            + self.get_bf_diff_b0_to_gap()
            - self.get_missing_b0_bf_sum()
        )

    @staticmethod
    def get_total_b0_belle_rescaled_bf() -> float:
        return (
            2 * belle_bf_b0_to_d_l_nu
            + belle_bf_b0_to_d_tau_nu
            + 2 * belle_bf_b0_to_d_st_l_nu
            + belle_bf_b0_to_d_st_tau_nu
            + sum_of_belle_b0_to_dss_l_nu_bfs
            + 2 * sum_of_belle_removed_b0_bfs
        )

    def get_mixed_correction_factor(self) -> float:
        _f = 0.5  # type: float
        return 1.0 + (_f * self.get_total_b0_bf_diff() / (1.0 - self.get_total_b0_belle_rescaled_bf()))


if __name__ == "__main__":
    print(f"BF_Belle - BF_Update for Dlnu               {MixedBBFComparison().get_bf_diff_b0_to_d_l_nu()}")
    print(f"BF_Belle - BF_Update for D*lnu              {MixedBBFComparison().get_bf_diff_b0_to_d_st_l_nu()}")
    print(f"BF_Belle - BF_Update for D**lnu             {MixedBBFComparison().get_bf_diff_b0_to_dss_l_nu()}")
    print(f"BF_Belle - BF_Update for Gap                {MixedBBFComparison().get_bf_diff_b0_to_gap()}")
    print(f"Missing in Simulation of sum of Updated BF  {MixedBBFComparison().get_missing_b0_bf_sum()}")
    print(f"BF_Belle - BF_Update total                  {MixedBBFComparison().get_total_b0_bf_diff()}")
    print(f"Mixed Scaling Factor                        {MixedBBFComparison().get_mixed_correction_factor()}")
