from typing import List, Tuple, Dict

from rdstar.offline_analysis.systematics import SysColsInfo
from rdstar.offline_analysis.reco_modes.reco_mode_utilities import RecoModeDictType
from rdstar.mc_reweighting.BtoDststlnu.BtoDststlnu_weights import BtoDststFFReweighting as BtoDss_FF
from rdstar.mc_reweighting.BtoDststlnu.BtoDststlnu_weights import DststType as BtoDststFF_decay_types

final_weight_column_for_asimov: str = "asimov_final_weights"

dss_modes: Dict[str, Tuple[int, ...]] = {
    "DZPiZ": (63, 64),
    "DMPiZ": (61, 62),
    "DSZPiZ": (67, 68),
    "DSMPiZ": (65, 66),
    "DZPiM": (71, 72),
    "DPPiM": (73, 74),
    "DSZPiM": (75, 76),
    "DSPPiM": (77, 78),
}

all_decay_modes: RecoModeDictType = {
    "DM": (11, 12),
    "DSM": (15, 16),
    "DZ": (13, 14),
    "DSZ": (17, 18),
    "DZPiZ": (63, 64),
    "DMPiZ": (61, 62),
    "DSZPiZ": (67, 68),
    "DSMPiZ": (65, 66),
    "DZPiM": (71, 72),
    "DPPiM": (73, 74),
    "DSZPiM": (75, 76),
    "DSPPiM": (77, 78),
}

_all_dss_modes: List[int] = []
for modes in dss_modes.values():
    _all_dss_modes.extend(list(modes))
all_dss_modes: Tuple[int, ...] = tuple(_all_dss_modes)

dss_labels: Dict[str, str] = {
    "DM": r"$B^0 \rightarrow D^- \ell^+$",
    "DSM": r"$B^0 \rightarrow D^{*-} \ell^+$",
    "DZ": r"$B^- \rightarrow D^0 \ell^-$",
    "DSZ": r"$B^- \rightarrow D^{*0} \ell^-$",
    "DZPiZ": r"$B^- \rightarrow D^0 \pi^0 \ell^-$",
    "DMPiZ": r"$B^0 \rightarrow D^- \pi^0 \ell^+$",
    "DSZPiZ": r"$B^- \rightarrow D^{*0} \pi^0 \ell^-$",
    "DSMPiZ": r"$B^0 \rightarrow D^{*-} \pi^0 \ell^+$",
    "DZPiM": r"$B^0 \rightarrow \overline{D}^{0} \pi^- \ell^+$",
    "DPPiM": r"$B^- \rightarrow D^{+} \pi^- \ell^-$",
    "DSZPiM": r"$B^0 \rightarrow \overline{D}^{*0} \pi^- \ell^+$",
    "DSPPiM": r"$B^- \rightarrow D^{*+} \pi^- \ell^-$",
    "all_modes": "All Reconstruction Modes",
}

dss_charged_B_neutral_pi_modes: Dict[str, List[int]] = {
    "DZ": [13, 14],
    "DSZ": [17, 18],
    "DZPiZ": [63, 64],
    "DSZPiZ": [67, 68],
}

dss_neutral_B_neutral_pi_modes: Dict[str, List[int]] = {
    "DM": [11, 12],
    "DSM": [15, 16],
    "DMPiZ": [61, 62],
    "DSMPiZ": [65, 66],
}

dss_charged_B_charged_pi_modes: Dict[str, List[int]] = {
    "DZ": [13, 14],
    "DSZ": [17, 18],
    "DPPiM": [73, 74],
    "DSPPiM": [77, 78],
}

dss_neutral_B_charged_pi_modes: Dict[str, List[int]] = {
    "DM": [11, 12],
    "DSM": [15, 16],
    "DZPiM": [71, 72],
    "DSZPiM": [75, 76],
}

dss_modes_for_DPiZ: Dict[str, List[int]] = {
    "DZPiZ": [63, 64],
    "DMPiZ": [61, 62],
}

dss_modes_for_DSPiZ: Dict[str, List[int]] = {
    "DSZPiZ": [67, 68],
    "DSMPiZ": [65, 66],
}

dss_modes_for_DPiP: Dict[str, List[int]] = {
    "DZPiM": [71, 72],
    "DPPiM": [73, 74],
}

dss_modes_for_DSPiP: Dict[str, List[int]] = {
    "DSZPiM": [75, 76],
    "DSPPiM": [77, 78],
}


# Mbc Sideband Fits
mbc_dss_charged_B_neutral_pi_modes: Dict[str, List[int]] = {
    "DZ": [13, 14],
    "DSZ": [17, 18],
}

mbc_dss_neutral_B_neutral_pi_modes: Dict[str, List[int]] = {
    "DM": [11, 12],
    "DSM": [15, 16],
}


# Normalization Only Fits
normalization_fit_dss_charged_B_modes: Dict[str, List[int]] = {
    "DZ": [13, 14],
    "DSZ": [17, 18],
}

normalization_fit_dss_neutral_B_modes: Dict[str, List[int]] = {
    "DM": [11, 12],
    "DSM": [15, 16],
}


dss_ff_systematics: List[SysColsInfo] = []
for dstst_decay_type in BtoDststFF_decay_types:
    dss_ff_systematics.append(
        SysColsInfo(
            var=BtoDss_FF.get_var_dstst_weight_name(dstst_type=dstst_decay_type.value),
            ratio=BtoDss_FF.get_nominal_dstst_weight_name(dstst_type=dstst_decay_type.value),
            from_df=True,
        )
    )
