import os

import pandas as pd
import numpy as np
import uncertainties as unc
import json
from analysistools.statistics.limits import binomial_limit

source_dir = "/storage/9/fmetzner/rdstar/data_set_prod_18thApr2019_stream0"


def get_branching_ratio(res):
    covariance = np.array([[res.params.covariance[2, 2], res.params.covariance[2, 3]],
                           [res.params.covariance[3, 2], res.params.covariance[3, 3]]])
    correlation = np.array([[res.params.correlation[2, 2], res.params.correlation[2, 3]],
                            [res.params.correlation[3, 2], res.params.correlation[3, 3]]])
    DS_yield = res.params["DSl_yield"][0]
    DSS_yield = res.params["DSSl_yield"][0]

    DS_effs, DS_after = get_efficienicies(
        [
            "DSMMu",
            "DSZMu",
            "DSZE",
            "DSME"
        ],
        4
    )

    DSS_effs, DSS_after = get_efficienicies(
        [
            'BZD1EEff',
            'BZD1MuEff',
            'BZD0sEEff',
            'BZDp1EEff',
            'BZD2sEEff',
            'BZD0sMuEff',
            'BZDp1MuEff',
            'BZD2sMuEff',
            'BMD1EEff',
            'BMD0sEEff',
            'BMDp1EEff',
            'BMD2sEEff',
            'BMD1MuEff',
            'BMD0sMuEff',
            'BMDp1MuEff',
            'BMD2sMuEff'
        ],
        5
    )

    uDS_yield, uDSS_yield = unc.correlated_values([DS_yield, DSS_yield], covariance)
    uDS_eff = unc.ufloat(DS_effs["nominal"], DS_effs["error"])
    uDSS_eff = unc.ufloat(DSS_effs["nominal"], DSS_effs["error"])
    uDS_br = (unc.ufloat(0.0533, 0.001) + unc.ufloat(0.0579, 0.001)) / 2
    # uDS_br = unc.ufloat(0.0579, 0.001)

    DSSl_br = uDS_br * (uDSS_yield / uDS_yield) * (uDS_eff / uDSS_eff)

    summary_dict = {
        "DSl_yield": "{}".format(uDS_yield),
        "DSSl_yield": "{}".format(uDSS_yield),
        "Cov": covariance.tolist(),
        "Corr": correlation.tolist(),
        "DSl_eff": "{}".format(uDS_eff),
        "DSSl_eff": "{}".format(uDSS_eff),
        "DSl_br": "{}".format(uDS_br),
        "DSSl_br": "{}".format(DSSl_br),
    }

    return summary_dict

    #     R_mc = 0.015893/0.0533
    #     return uDS_br*(uDSS_yield/uDS_yield)*R_mc*DS_after/DSS_after

    # br = uDSS_yield/(uDSS_eff*2*2*772e6)
    # return br


def get_efficienicies(samples, ID, sideband=False):
    eff_file_path = "/storage/9/fmetzner/rdstar/data_set_prod_18thApr2019_stream0/Efficiencies/efficiency_dict.json"
    with open(eff_file_path) as infile:
        eff_df = pd.DataFrame(json.load(infile))

    before = np.sum(eff_df.loc["before", samples])
    # before = np.sum(np.full(len(samples), 2500000))

    main_appendix = "_Y4S_ntuple_first_selection_with_CS_CS1.h5"
    mbc_sideband_appendix = "_Y4S_ntuple_first_selection_with_CS_CS1_mbc_sideband.h5"

    if sideband:
        df = pd.concat([pd.read_hdf(os.path.join(source_dir, s, f"{s}{mbc_sideband_appendix}")) for s in samples])
    else:
        df = pd.concat([pd.read_hdf(os.path.join(source_dir, s, f"{s}{main_appendix}")) for s in samples])

    dstst_cond = df.daughter__bo1__cm__spextraInfo__bodecayModeID__bc__bc.isin([61, 62, 63, 64, 65, 66, 67, 68])
    dstst_df = df.loc[dstst_cond].query(f"SIG_ID=={ID}")

    if sideband:
        sig_after = len(dstst_df.loc[(-2.5 < dstst_df.m2RecoilSignalSide) & (dstst_df.m2RecoilSignalSide < 7.0)].index)
    else:
        sig_after = len(dstst_df.loc[(-2.5 < dstst_df.m2RecoilSignalSide) & (dstst_df.m2RecoilSignalSide < 5.0)].index)

    # sig_after = np.sum(eff_df.loc["sig_dstst_after", samples])

    sig_eff = {}
    sig_eff["nominal"] = sig_after / before
    errors = binomial_limit(before, sig_after)
    sig_eff["up"] = errors[0]
    sig_eff["down"] = errors[1]
    sig_eff["error"] = (np.abs(sig_eff["nominal"] - sig_eff["up"]) + np.abs(sig_eff["nominal"] - sig_eff["down"])) / 2

    return sig_eff, sig_after
