import os
import numpy as np
import pandas as pd

from rdstar.offline_analysis.rdstar_samples import alternative_sample_keys

from rdstar.offline_analysis.fitting.dedicated_fit_approach.fitter_rdstar import RDStarFitter
from rdstar.offline_analysis.fitting.dedicated_fit_approach.fit_setups import FitSetupCollection


def get_file_path_from_key(key, q2_sb):
    sb_tag = "_q2_sideband" if q2_sb else ""
    file_name = f"{key}_withFinalWeights_withMMSQResCorr_{selection_name}_with_CS_CS1{sb_tag}.h5"
    file_path = os.path.join(base_input_path, file_name)
    assert os.path.exists(file_path), (key, file_path)
    return file_path


if __name__ == "__main__":

    selection_name = "optimized_mva_selection_sec_sel_mva"
    base_input_path = os.path.join(
        # "/ceph/fmetzner/rdstar/data_set_prod_24thMarch2022_stream0/CombinedSamples/",
        "/storage/9/fmetzner/ceph_rdstar_backup/rdstar/data_set_prod_24thMarch2022_stream0/CombinedSamples/",
        selection_name,
    )

    mc_paths = [get_file_path_from_key(key=sample_key, q2_sb=False) for sample_key in alternative_sample_keys]
    mc_sample = pd.concat([pd.read_hdf(mc_path) for mc_path in mc_paths], ignore_index=True, sort=False)
    mc_sample.reset_index(inplace=True)

    rdstar_asimov_fit_setup = FitSetupCollection.rdstar_asimov_fit_setup
    rdstar_asimov_fit_setup.prepare_mc_df(df=mc_sample)

    print("Setting up fitter...")
    fitter = RDStarFitter(
        fit_setup_info=rdstar_asimov_fit_setup,
        mc_df=mc_sample,
        data_df=None,
    )

    initial_params_vector: np.ndarray = np.copy(fitter.initial_parameters)
    assert len(initial_params_vector) == fitter.end_of_floating_params, (
        initial_params_vector,
        fitter.initial_parameters,
        fitter.end_of_floating_params,
    )

    print(initial_params_vector)

    print(fitter.cost_fcn(params=initial_params_vector))
