"""
MC SigID groups for extended mc matching information.

Felix Metzner 2023
"""

from typing import Tuple, Dict, List

from rdstar.offline_analysis.mc_matching.mc_matching_ids import MCSigIDs


__all__ = [
    "MCMatchingGroups",
]


class MCMatchingGroups:
    all_sig_ids: Tuple[int, ...] = MCSigIDs.get_all_sig_ids()

    # fmt: off
    # B -> D tau nu
    group_Dtaunu_via_e: Tuple[int, ...] = (MCSigIDs.cB_to_nD_tau_via_e_nu.id, MCSigIDs.nB_to_cD_tau_via_e_nu.id)
    group_Dtaunu_via_mu: Tuple[int, ...] = (MCSigIDs.cB_to_nD_tau_via_mu_nu.id, MCSigIDs.nB_to_cD_tau_via_mu_nu.id)
    group_neutral_Dtaunu: Tuple[int, ...] = (MCSigIDs.cB_to_nD_tau_via_e_nu.id, MCSigIDs.cB_to_nD_tau_via_mu_nu.id)
    group_charged_Dtaunu: Tuple[int, ...] = (MCSigIDs.nB_to_cD_tau_via_e_nu.id, MCSigIDs.nB_to_cD_tau_via_mu_nu.id)
    group_Dtaunu: Tuple[int, ...] = group_Dtaunu_via_e + group_Dtaunu_via_mu

    # B -> D* tau nu
    group_DStaunu_via_e_via_pi: Tuple[int, ...] = (MCSigIDs.cB_to_nDst_tau_via_e_nu_with_Dst_to_pi.id, MCSigIDs.nB_to_cDst_tau_via_e_nu_with_Dst_to_pi.id)
    group_DStaunu_via_e_via_ph: Tuple[int, ...] = (MCSigIDs.cB_to_nDst_tau_via_e_nu_with_Dst_to_photon.id, MCSigIDs.nB_to_cDst_tau_via_e_nu_with_Dst_to_photon.id)
    group_DStaunu_via_mu_via_pi: Tuple[int, ...] = (MCSigIDs.cB_to_nDst_tau_via_mu_nu_with_Dst_to_pi.id, MCSigIDs.nB_to_cDst_tau_via_mu_nu_with_Dst_to_pi.id)
    group_DStaunu_via_mu_via_ph: Tuple[int, ...] = (MCSigIDs.cB_to_nDst_tau_via_mu_nu_with_Dst_to_photon.id, MCSigIDs.nB_to_cDst_tau_via_mu_nu_with_Dst_to_photon.id)
    group_DStaunu_via_e: Tuple[int, ...] = group_DStaunu_via_e_via_pi + group_DStaunu_via_e_via_ph
    group_DStaunu_via_mu: Tuple[int, ...] = group_DStaunu_via_mu_via_pi + group_DStaunu_via_mu_via_ph
    group_DStaunu_via_pi: Tuple[int, ...] = group_DStaunu_via_e_via_pi + group_DStaunu_via_mu_via_pi
    group_DStaunu_via_photon: Tuple[int, ...] = group_DStaunu_via_e_via_ph + group_DStaunu_via_mu_via_ph
    group_charged_DStaunu_via_e: Tuple[int, ...] = (MCSigIDs.nB_to_cDst_tau_via_e_nu_with_Dst_to_pi.id, MCSigIDs.nB_to_cDst_tau_via_e_nu_with_Dst_to_photon.id)
    group_neutral_DStaunu_via_e: Tuple[int, ...] = (MCSigIDs.cB_to_nDst_tau_via_e_nu_with_Dst_to_pi.id, MCSigIDs.cB_to_nDst_tau_via_e_nu_with_Dst_to_photon.id)
    group_charged_DStaunu_via_mu: Tuple[int, ...] = (MCSigIDs.nB_to_cDst_tau_via_mu_nu_with_Dst_to_pi.id, MCSigIDs.nB_to_cDst_tau_via_mu_nu_with_Dst_to_photon.id)
    group_neutral_DStaunu_via_mu: Tuple[int, ...] = (MCSigIDs.cB_to_nDst_tau_via_mu_nu_with_Dst_to_pi.id, MCSigIDs.cB_to_nDst_tau_via_mu_nu_with_Dst_to_photon.id)
    group_charged_DStaunu: Tuple[int, ...] = group_charged_DStaunu_via_e + group_charged_DStaunu_via_mu
    group_neutral_DStaunu: Tuple[int, ...] = group_neutral_DStaunu_via_e + group_neutral_DStaunu_via_mu
    group_charged_DStaunu_via_pi: Tuple[int, ...] = (MCSigIDs.nB_to_cDst_tau_via_e_nu_with_Dst_to_pi.id, MCSigIDs.nB_to_cDst_tau_via_mu_nu_with_Dst_to_pi.id)
    group_neutral_DStaunu_via_pi: Tuple[int, ...] = (MCSigIDs.cB_to_nDst_tau_via_e_nu_with_Dst_to_pi.id, MCSigIDs.cB_to_nDst_tau_via_mu_nu_with_Dst_to_pi.id)
    group_charged_DStaunu_via_photon: Tuple[int, ...] = (MCSigIDs.nB_to_cDst_tau_via_e_nu_with_Dst_to_photon.id, MCSigIDs.nB_to_cDst_tau_via_mu_nu_with_Dst_to_photon.id)
    group_neutral_DStaunu_via_photon: Tuple[int, ...] = (MCSigIDs.cB_to_nDst_tau_via_e_nu_with_Dst_to_photon.id, MCSigIDs.cB_to_nDst_tau_via_mu_nu_with_Dst_to_photon.id)
    group_DStaunu: Tuple[int, ...] = group_DStaunu_via_e + group_DStaunu_via_mu

    # B -> D l nu
    group_Dlnu_via_e: Tuple[int, ...] = (MCSigIDs.cB_to_nD_e_nu.id, MCSigIDs.nB_to_cD_e_nu.id)
    group_Dlnu_via_mu: Tuple[int, ...] = (MCSigIDs.cB_to_nD_mu_nu.id, MCSigIDs.nB_to_cD_mu_nu.id)
    group_neutral_Dlnu: Tuple[int, ...] = (MCSigIDs.cB_to_nD_e_nu.id, MCSigIDs.cB_to_nD_mu_nu.id)
    group_charged_Dlnu: Tuple[int, ...] = (MCSigIDs.nB_to_cD_e_nu.id, MCSigIDs.nB_to_cD_mu_nu.id)
    group_Dlnu: Tuple[int, ...] = group_Dlnu_via_e + group_Dlnu_via_mu

    # Special sub-groups for individual sub-categories:
    group_chargedD_enu: Tuple[int, ...] = (MCSigIDs.nB_to_cD_e_nu.id, )
    group_chargedD_munu: Tuple[int, ...] = (MCSigIDs.nB_to_cD_mu_nu.id, )
    group_neutralD_enu: Tuple[int, ...] = (MCSigIDs.cB_to_nD_e_nu.id, )
    group_neutralD_munu: Tuple[int, ...] = (MCSigIDs.cB_to_nD_mu_nu.id, )

    # B -> D* l nu
    group_DSlnu_via_e_via_pi: Tuple[int, ...] = (MCSigIDs.cB_to_nDst_e_nu_with_Dst_to_pi.id, MCSigIDs.nB_to_cDst_e_nu_with_Dst_to_pi.id)
    group_DSlnu_via_e_via_ph: Tuple[int, ...] = (MCSigIDs.cB_to_nDst_e_nu_with_Dst_to_photon.id, MCSigIDs.nB_to_cDst_e_nu_with_Dst_to_photon.id)
    group_DSlnu_via_mu_via_pi: Tuple[int, ...] = (MCSigIDs.cB_to_nDst_mu_nu_with_Dst_to_pi.id, MCSigIDs.nB_to_cDst_mu_nu_with_Dst_to_pi.id)
    group_DSlnu_via_mu_via_ph: Tuple[int, ...] = (MCSigIDs.cB_to_nDst_mu_nu_with_Dst_to_photon.id, MCSigIDs.nB_to_cDst_mu_nu_with_Dst_to_photon.id)
    group_DSlnu_via_e: Tuple[int, ...] = group_DSlnu_via_e_via_pi + group_DSlnu_via_e_via_ph
    group_DSlnu_via_mu: Tuple[int, ...] = group_DSlnu_via_mu_via_pi + group_DSlnu_via_mu_via_ph
    group_DSlnu_via_pi: Tuple[int, ...] = group_DSlnu_via_e_via_pi + group_DSlnu_via_mu_via_pi
    group_DSlnu_via_ph: Tuple[int, ...] = group_DSlnu_via_e_via_ph + group_DSlnu_via_mu_via_ph
    group_charged_DSlnu_via_e: Tuple[int, ...] = (MCSigIDs.nB_to_cDst_e_nu_with_Dst_to_pi.id, MCSigIDs.nB_to_cDst_e_nu_with_Dst_to_photon.id)
    group_neutral_DSlnu_via_e: Tuple[int, ...] = (MCSigIDs.cB_to_nDst_e_nu_with_Dst_to_pi.id, MCSigIDs.cB_to_nDst_e_nu_with_Dst_to_photon.id)
    group_charged_DSlnu_via_mu: Tuple[int, ...] = (MCSigIDs.nB_to_cDst_mu_nu_with_Dst_to_pi.id, MCSigIDs.nB_to_cDst_mu_nu_with_Dst_to_photon.id)
    group_neutral_DSlnu_via_mu: Tuple[int, ...] = (MCSigIDs.cB_to_nDst_mu_nu_with_Dst_to_pi.id, MCSigIDs.cB_to_nDst_mu_nu_with_Dst_to_photon.id)
    group_charged_DSlnu: Tuple[int, ...] = group_charged_DSlnu_via_e + group_charged_DSlnu_via_mu
    group_neutral_DSlnu: Tuple[int, ...] = group_neutral_DSlnu_via_e + group_neutral_DSlnu_via_mu
    group_charged_DSlnu_via_pi: Tuple[int, ...] = (MCSigIDs.nB_to_cDst_e_nu_with_Dst_to_pi.id, MCSigIDs.nB_to_cDst_mu_nu_with_Dst_to_pi.id)
    group_neutral_DSlnu_via_pi: Tuple[int, ...] = (MCSigIDs.cB_to_nDst_e_nu_with_Dst_to_pi.id, MCSigIDs.cB_to_nDst_mu_nu_with_Dst_to_pi.id)
    group_charged_DSlnu_via_photon: Tuple[int, ...] = (MCSigIDs.nB_to_cDst_e_nu_with_Dst_to_photon.id, MCSigIDs.nB_to_cDst_mu_nu_with_Dst_to_photon.id)
    group_neutral_DSlnu_via_photon: Tuple[int, ...] = (MCSigIDs.cB_to_nDst_e_nu_with_Dst_to_photon.id, MCSigIDs.cB_to_nDst_mu_nu_with_Dst_to_photon.id)
    group_DSlnu: Tuple[int, ...] = group_DSlnu_via_e + group_DSlnu_via_mu

    # Special sub-groups for individual sub-categories:
    group_chargedDS_e_nu: Tuple[int, ...] = group_charged_DSlnu_via_e
    group_chargedDS_mu_nu: Tuple[int, ...] = group_charged_DSlnu_via_mu
    group_neutralDS_e_nu_DS_via_pi: Tuple[int, ...] = (MCSigIDs.cB_to_nDst_e_nu_with_Dst_to_pi.id, )
    group_neutralDS_mu_nu_DS_via_pi: Tuple[int, ...] = (MCSigIDs.cB_to_nDst_mu_nu_with_Dst_to_pi.id, )
    group_neutralDS_e_nu_DS_via_photon: Tuple[int, ...] = (MCSigIDs.cB_to_nDst_e_nu_with_Dst_to_photon.id, )
    group_neutralDS_mu_nu_DS_via_photon: Tuple[int, ...] = (MCSigIDs.cB_to_nDst_mu_nu_with_Dst_to_photon.id, )

    # B -> D** (-> D(*) pi0) l nu
    group_DSSlnu_via_pi0_with_Done_via_e: Tuple[int, ...] = (MCSigIDs.cB_to_nDone_via_pi0_e_nu.id, MCSigIDs.nB_to_cDone_via_pi0_e_nu.id)
    group_DSSlnu_via_pi0_with_Done_via_mu: Tuple[int, ...] = (MCSigIDs.cB_to_nDone_via_pi0_mu_nu.id, MCSigIDs.nB_to_cDone_via_pi0_mu_nu.id)
    group_DSSlnu_via_pi0_with_Done_via_tau: Tuple[int, ...] = (MCSigIDs.cB_to_nDone_via_pi0_tau_nu.id, MCSigIDs.nB_to_cDone_via_pi0_tau_nu.id)
    group_DSSlnu_via_pi0_with_Dtwostar_via_e: Tuple[int, ...] = (MCSigIDs.cB_to_nDtwostar_via_pi0_e_nu.id, MCSigIDs.nB_to_cDtwostar_via_pi0_e_nu.id)
    group_DSSlnu_via_pi0_with_Dtwostar_via_mu: Tuple[int, ...] = (MCSigIDs.cB_to_nDtwostar_via_pi0_mu_nu.id, MCSigIDs.nB_to_cDtwostar_via_pi0_mu_nu.id)
    group_DSSlnu_via_pi0_with_Dtwostar_via_tau: Tuple[int, ...] = (MCSigIDs.cB_to_nDtwostar_via_pi0_tau_nu.id, MCSigIDs.nB_to_cDtwostar_via_pi0_tau_nu.id)
    group_DSSlnu_via_pi0_with_Dzerostar_via_e: Tuple[int, ...] = (MCSigIDs.cB_to_nDzerostar_via_pi0_e_nu.id, MCSigIDs.nB_to_cDzerostar_via_pi0_e_nu.id)
    group_DSSlnu_via_pi0_with_Dzerostar_via_mu: Tuple[int, ...] = (MCSigIDs.cB_to_nDzerostar_via_pi0_mu_nu.id, MCSigIDs.nB_to_cDzerostar_via_pi0_mu_nu.id)
    group_DSSlnu_via_pi0_with_Dzerostar_via_tau: Tuple[int, ...] = (MCSigIDs.cB_to_nDzerostar_via_pi0_tau_nu.id, MCSigIDs.nB_to_cDzerostar_via_pi0_tau_nu.id)
    group_DSSlnu_via_pi0_with_Dprimeone_via_e: Tuple[int, ...] = (MCSigIDs.cB_to_nDprimeone_via_pi0_e_nu.id, MCSigIDs.nB_to_cDprimeone_via_pi0_e_nu.id)
    group_DSSlnu_via_pi0_with_Dprimeone_via_mu: Tuple[int, ...] = (MCSigIDs.cB_to_nDprimeone_via_pi0_mu_nu.id, MCSigIDs.nB_to_cDprimeone_via_pi0_mu_nu.id)
    group_DSSlnu_via_pi0_with_Dprimeone_via_tau: Tuple[int, ...] = (MCSigIDs.cB_to_nDprimeone_via_pi0_tau_nu.id, MCSigIDs.nB_to_cDprimeone_via_pi0_tau_nu.id)

    group_DSSlnu_via_pi0_with_Done: Tuple[int, ...] = group_DSSlnu_via_pi0_with_Done_via_e + group_DSSlnu_via_pi0_with_Done_via_mu + group_DSSlnu_via_pi0_with_Done_via_tau
    group_DSSlnu_via_pi0_with_Dtwostar: Tuple[int, ...] = group_DSSlnu_via_pi0_with_Dtwostar_via_e + group_DSSlnu_via_pi0_with_Dtwostar_via_mu + group_DSSlnu_via_pi0_with_Dtwostar_via_tau
    group_DSSlnu_via_pi0_with_Dzerostar: Tuple[int, ...] = group_DSSlnu_via_pi0_with_Dzerostar_via_e + group_DSSlnu_via_pi0_with_Dzerostar_via_mu + group_DSSlnu_via_pi0_with_Dzerostar_via_tau
    group_DSSlnu_via_pi0_with_Dprimeone: Tuple[int, ...] = group_DSSlnu_via_pi0_with_Dprimeone_via_e + group_DSSlnu_via_pi0_with_Dprimeone_via_mu + group_DSSlnu_via_pi0_with_Dprimeone_via_tau

    group_narrow_charged_DSSlnu_via_pi0_via_e: Tuple[int, ...] = (MCSigIDs.nB_to_cDone_via_pi0_e_nu.id, MCSigIDs.nB_to_cDtwostar_via_pi0_e_nu.id)
    group_wide_charged_DSSlnu_via_pi0_via_e: Tuple[int, ...] = (MCSigIDs.nB_to_cDzerostar_via_pi0_e_nu.id, MCSigIDs.nB_to_cDprimeone_via_pi0_e_nu.id)
    group_narrow_neutral_DSSlnu_via_pi0_via_e: Tuple[int, ...] = (MCSigIDs.cB_to_nDone_via_pi0_e_nu.id, MCSigIDs.cB_to_nDtwostar_via_pi0_e_nu.id)
    group_wide_neutral_DSSlnu_via_pi0_via_e: Tuple[int, ...] = (MCSigIDs.cB_to_nDzerostar_via_pi0_e_nu.id, MCSigIDs.cB_to_nDprimeone_via_pi0_e_nu.id)
    group_narrow_charged_DSSlnu_via_pi0_via_mu: Tuple[int, ...] = (MCSigIDs.nB_to_cDone_via_pi0_mu_nu.id, MCSigIDs.nB_to_cDtwostar_via_pi0_mu_nu.id)
    group_wide_charged_DSSlnu_via_pi0_via_mu: Tuple[int, ...] = (MCSigIDs.nB_to_cDzerostar_via_pi0_mu_nu.id, MCSigIDs.nB_to_cDprimeone_via_pi0_mu_nu.id)
    group_narrow_neutral_DSSlnu_via_pi0_via_mu: Tuple[int, ...] = (MCSigIDs.cB_to_nDone_via_pi0_mu_nu.id, MCSigIDs.cB_to_nDtwostar_via_pi0_mu_nu.id)
    group_wide_neutral_DSSlnu_via_pi0_via_mu: Tuple[int, ...] = (MCSigIDs.cB_to_nDzerostar_via_pi0_mu_nu.id, MCSigIDs.cB_to_nDprimeone_via_pi0_mu_nu.id)
    group_narrow_charged_DSSlnu_via_pi0_via_tau: Tuple[int, ...] = (MCSigIDs.nB_to_cDone_via_pi0_tau_nu.id, MCSigIDs.nB_to_cDtwostar_via_pi0_tau_nu.id)
    group_wide_charged_DSSlnu_via_pi0_via_tau: Tuple[int, ...] = (MCSigIDs.nB_to_cDzerostar_via_pi0_tau_nu.id, MCSigIDs.nB_to_cDprimeone_via_pi0_tau_nu.id)
    group_narrow_neutral_DSSlnu_via_pi0_via_tau: Tuple[int, ...] = (MCSigIDs.cB_to_nDone_via_pi0_tau_nu.id, MCSigIDs.cB_to_nDtwostar_via_pi0_tau_nu.id)
    group_wide_neutral_DSSlnu_via_pi0_via_tau: Tuple[int, ...] = (MCSigIDs.cB_to_nDzerostar_via_pi0_tau_nu.id, MCSigIDs.cB_to_nDprimeone_via_pi0_tau_nu.id)

    group_charged_DSSlnu_via_pi0_via_e: Tuple[int, ...] = group_narrow_charged_DSSlnu_via_pi0_via_e + group_wide_charged_DSSlnu_via_pi0_via_e
    group_neutral_DSSlnu_via_pi0_via_e: Tuple[int, ...] = group_narrow_neutral_DSSlnu_via_pi0_via_e + group_wide_neutral_DSSlnu_via_pi0_via_e
    group_charged_DSSlnu_via_pi0_via_mu: Tuple[int, ...] = group_narrow_charged_DSSlnu_via_pi0_via_mu + group_wide_charged_DSSlnu_via_pi0_via_mu
    group_neutral_DSSlnu_via_pi0_via_mu: Tuple[int, ...] = group_narrow_neutral_DSSlnu_via_pi0_via_mu + group_wide_neutral_DSSlnu_via_pi0_via_mu
    group_charged_DSSlnu_via_pi0_via_tau: Tuple[int, ...] = group_narrow_charged_DSSlnu_via_pi0_via_tau + group_wide_charged_DSSlnu_via_pi0_via_tau
    group_neutral_DSSlnu_via_pi0_via_tau: Tuple[int, ...] = group_narrow_neutral_DSSlnu_via_pi0_via_tau + group_wide_neutral_DSSlnu_via_pi0_via_tau

    group_narrow_DSSlnu_via_pi0_via_e: Tuple[int, ...] = group_narrow_charged_DSSlnu_via_pi0_via_e + group_narrow_neutral_DSSlnu_via_pi0_via_e
    group_wide_DSSlnu_via_pi0_via_e: Tuple[int, ...] = group_wide_charged_DSSlnu_via_pi0_via_e + group_wide_neutral_DSSlnu_via_pi0_via_e
    group_narrow_DSSlnu_via_pi0_via_mu: Tuple[int, ...] = group_narrow_charged_DSSlnu_via_pi0_via_mu + group_narrow_neutral_DSSlnu_via_pi0_via_mu
    group_wide_DSSlnu_via_pi0_via_mu: Tuple[int, ...] = group_wide_charged_DSSlnu_via_pi0_via_mu + group_wide_neutral_DSSlnu_via_pi0_via_mu
    group_narrow_DSSlnu_via_pi0_via_tau: Tuple[int, ...] = group_narrow_charged_DSSlnu_via_pi0_via_tau + group_narrow_neutral_DSSlnu_via_pi0_via_tau
    group_wide_DSSlnu_via_pi0_via_tau: Tuple[int, ...] = group_wide_charged_DSSlnu_via_pi0_via_tau + group_wide_neutral_DSSlnu_via_pi0_via_tau

    group_DSSlnu_via_pi0_via_e: Tuple[int, ...] = group_narrow_DSSlnu_via_pi0_via_e + group_wide_DSSlnu_via_pi0_via_e
    group_DSSlnu_via_pi0_via_mu: Tuple[int, ...] = group_narrow_DSSlnu_via_pi0_via_mu + group_wide_DSSlnu_via_pi0_via_mu
    group_DSSlnu_via_pi0_via_tau: Tuple[int, ...] = group_narrow_DSSlnu_via_pi0_via_tau + group_wide_DSSlnu_via_pi0_via_tau

    group_charged_DSSlnu_via_pi0_with_Done: Tuple[int, ...] = (MCSigIDs.nB_to_cDone_via_pi0_e_nu.id, MCSigIDs.nB_to_cDone_via_pi0_mu_nu.id, MCSigIDs.nB_to_cDone_via_pi0_tau_nu.id)
    group_neutral_DSSlnu_via_pi0_with_Done: Tuple[int, ...] = (MCSigIDs.cB_to_nDone_via_pi0_e_nu.id, MCSigIDs.cB_to_nDone_via_pi0_mu_nu.id, MCSigIDs.cB_to_nDone_via_pi0_tau_nu.id)
    group_charged_DSSlnu_via_pi0_with_Dtwostar: Tuple[int, ...] = (MCSigIDs.nB_to_cDtwostar_via_pi0_e_nu.id, MCSigIDs.nB_to_cDtwostar_via_pi0_mu_nu.id, MCSigIDs.nB_to_cDtwostar_via_pi0_tau_nu.id)
    group_neutral_DSSlnu_via_pi0_with_Dtwostar: Tuple[int, ...] = (MCSigIDs.cB_to_nDtwostar_via_pi0_e_nu.id, MCSigIDs.cB_to_nDtwostar_via_pi0_mu_nu.id, MCSigIDs.cB_to_nDtwostar_via_pi0_tau_nu.id)
    group_charged_DSSlnu_via_pi0_with_Dzerostar: Tuple[int, ...] = (MCSigIDs.nB_to_cDzerostar_via_pi0_e_nu.id, MCSigIDs.nB_to_cDzerostar_via_pi0_mu_nu.id, MCSigIDs.nB_to_cDzerostar_via_pi0_tau_nu.id)
    group_neutral_DSSlnu_via_pi0_with_Dzerostar: Tuple[int, ...] = (MCSigIDs.cB_to_nDzerostar_via_pi0_e_nu.id, MCSigIDs.cB_to_nDzerostar_via_pi0_mu_nu.id, MCSigIDs.cB_to_nDzerostar_via_pi0_tau_nu.id)
    group_charged_DSSlnu_via_pi0_with_Dprimeone: Tuple[int, ...] = (MCSigIDs.nB_to_cDprimeone_via_pi0_e_nu.id, MCSigIDs.nB_to_cDprimeone_via_pi0_mu_nu.id, MCSigIDs.nB_to_cDprimeone_via_pi0_tau_nu.id)
    group_neutral_DSSlnu_via_pi0_with_Dprimeone: Tuple[int, ...] = (MCSigIDs.cB_to_nDprimeone_via_pi0_e_nu.id, MCSigIDs.cB_to_nDprimeone_via_pi0_mu_nu.id, MCSigIDs.cB_to_nDprimeone_via_pi0_tau_nu.id)

    group_narrow_charged_DSSlnu_via_pi0: Tuple[int, ...] = group_charged_DSSlnu_via_pi0_with_Done + group_charged_DSSlnu_via_pi0_with_Dtwostar
    group_narrow_neutral_DSSlnu_via_pi0: Tuple[int, ...] = group_neutral_DSSlnu_via_pi0_with_Done + group_neutral_DSSlnu_via_pi0_with_Dtwostar
    group_wide_charged_DSSlnu_via_pi0: Tuple[int, ...] = group_charged_DSSlnu_via_pi0_with_Dzerostar + group_charged_DSSlnu_via_pi0_with_Dprimeone
    group_wide_neutral_DSSlnu_via_pi0: Tuple[int, ...] = group_neutral_DSSlnu_via_pi0_with_Dzerostar + group_neutral_DSSlnu_via_pi0_with_Dprimeone

    group_charged_DSSlnu_via_pi0: Tuple[int, ...] = group_narrow_charged_DSSlnu_via_pi0 + group_wide_charged_DSSlnu_via_pi0
    group_neutral_DSSlnu_via_pi0: Tuple[int, ...] = group_narrow_neutral_DSSlnu_via_pi0 + group_wide_neutral_DSSlnu_via_pi0

    group_narrow_DSSlnu_via_pi0: Tuple[int, ...] = group_DSSlnu_via_pi0_with_Done + group_DSSlnu_via_pi0_with_Dtwostar
    group_wide_DSSlnu_via_pi0: Tuple[int, ...] = group_DSSlnu_via_pi0_with_Dzerostar + group_DSSlnu_via_pi0_with_Dprimeone

    group_DSSlnu_via_pi0: Tuple[int, ...] = group_narrow_DSSlnu_via_pi0 + group_wide_DSSlnu_via_pi0

    # B -> D** (-> D(*) pi+) l nu
    group_DSSlnu_via_pip_with_Done_via_e: Tuple[int, ...] = (MCSigIDs.cB_to_nDone_via_pip_e_nu.id, MCSigIDs.nB_to_cDone_via_pip_e_nu.id)
    group_DSSlnu_via_pip_with_Done_via_mu: Tuple[int, ...] = (MCSigIDs.cB_to_nDone_via_pip_mu_nu.id, MCSigIDs.nB_to_cDone_via_pip_mu_nu.id)
    group_DSSlnu_via_pip_with_Done_via_tau: Tuple[int, ...] = (MCSigIDs.cB_to_nDone_via_pip_tau_nu.id, MCSigIDs.nB_to_cDone_via_pip_tau_nu.id)
    group_DSSlnu_via_pip_with_Dtwostar_via_e: Tuple[int, ...] = (MCSigIDs.cB_to_nDtwostar_via_pip_e_nu.id, MCSigIDs.nB_to_cDtwostar_via_pip_e_nu.id)
    group_DSSlnu_via_pip_with_Dtwostar_via_mu: Tuple[int, ...] = (MCSigIDs.cB_to_nDtwostar_via_pip_mu_nu.id, MCSigIDs.nB_to_cDtwostar_via_pip_mu_nu.id)
    group_DSSlnu_via_pip_with_Dtwostar_via_tau: Tuple[int, ...] = (MCSigIDs.cB_to_nDtwostar_via_pip_tau_nu.id, MCSigIDs.nB_to_cDtwostar_via_pip_tau_nu.id)
    group_DSSlnu_via_pip_with_Dzerostar_via_e: Tuple[int, ...] = (MCSigIDs.cB_to_nDzerostar_via_pip_e_nu.id, MCSigIDs.nB_to_cDzerostar_via_pip_e_nu.id)
    group_DSSlnu_via_pip_with_Dzerostar_via_mu: Tuple[int, ...] = (MCSigIDs.cB_to_nDzerostar_via_pip_mu_nu.id, MCSigIDs.nB_to_cDzerostar_via_pip_mu_nu.id)
    group_DSSlnu_via_pip_with_Dzerostar_via_tau: Tuple[int, ...] = (MCSigIDs.cB_to_nDzerostar_via_pip_tau_nu.id, MCSigIDs.nB_to_cDzerostar_via_pip_tau_nu.id)
    group_DSSlnu_via_pip_with_Dprimeone_via_e: Tuple[int, ...] = (MCSigIDs.cB_to_nDprimeone_via_pip_e_nu.id, MCSigIDs.nB_to_cDprimeone_via_pip_e_nu.id)
    group_DSSlnu_via_pip_with_Dprimeone_via_mu: Tuple[int, ...] = (MCSigIDs.cB_to_nDprimeone_via_pip_mu_nu.id, MCSigIDs.nB_to_cDprimeone_via_pip_mu_nu.id)
    group_DSSlnu_via_pip_with_Dprimeone_via_tau: Tuple[int, ...] = (MCSigIDs.cB_to_nDprimeone_via_pip_tau_nu.id, MCSigIDs.nB_to_cDprimeone_via_pip_tau_nu.id)

    group_DSSlnu_via_pip_with_Done: Tuple[int, ...] = group_DSSlnu_via_pip_with_Done_via_e + group_DSSlnu_via_pip_with_Done_via_mu + group_DSSlnu_via_pip_with_Done_via_tau
    group_DSSlnu_via_pip_with_Dtwostar: Tuple[int, ...] = group_DSSlnu_via_pip_with_Dtwostar_via_e + group_DSSlnu_via_pip_with_Dtwostar_via_mu + group_DSSlnu_via_pip_with_Dtwostar_via_tau
    group_DSSlnu_via_pip_with_Dzerostar: Tuple[int, ...] = group_DSSlnu_via_pip_with_Dzerostar_via_e + group_DSSlnu_via_pip_with_Dzerostar_via_mu + group_DSSlnu_via_pip_with_Dzerostar_via_tau
    group_DSSlnu_via_pip_with_Dprimeone: Tuple[int, ...] = group_DSSlnu_via_pip_with_Dprimeone_via_e + group_DSSlnu_via_pip_with_Dprimeone_via_mu + group_DSSlnu_via_pip_with_Dprimeone_via_tau

    group_narrow_charged_DSSlnu_via_pip_via_e: Tuple[int, ...] = (MCSigIDs.nB_to_cDone_via_pip_e_nu.id, MCSigIDs.nB_to_cDtwostar_via_pip_e_nu.id)
    group_wide_charged_DSSlnu_via_pip_via_e: Tuple[int, ...] = (MCSigIDs.nB_to_cDzerostar_via_pip_e_nu.id, MCSigIDs.nB_to_cDprimeone_via_pip_e_nu.id)
    group_narrow_neutral_DSSlnu_via_pip_via_e: Tuple[int, ...] = (MCSigIDs.cB_to_nDone_via_pip_e_nu.id, MCSigIDs.cB_to_nDtwostar_via_pip_e_nu.id)
    group_wide_neutral_DSSlnu_via_pip_via_e: Tuple[int, ...] = (MCSigIDs.cB_to_nDzerostar_via_pip_e_nu.id, MCSigIDs.cB_to_nDprimeone_via_pip_e_nu.id)
    group_narrow_charged_DSSlnu_via_pip_via_mu: Tuple[int, ...] = (MCSigIDs.nB_to_cDone_via_pip_mu_nu.id, MCSigIDs.nB_to_cDtwostar_via_pip_mu_nu.id)
    group_wide_charged_DSSlnu_via_pip_via_mu: Tuple[int, ...] = (MCSigIDs.nB_to_cDzerostar_via_pip_mu_nu.id, MCSigIDs.nB_to_cDprimeone_via_pip_mu_nu.id)
    group_narrow_neutral_DSSlnu_via_pip_via_mu: Tuple[int, ...] = (MCSigIDs.cB_to_nDone_via_pip_mu_nu.id, MCSigIDs.cB_to_nDtwostar_via_pip_mu_nu.id)
    group_wide_neutral_DSSlnu_via_pip_via_mu: Tuple[int, ...] = (MCSigIDs.cB_to_nDzerostar_via_pip_mu_nu.id, MCSigIDs.cB_to_nDprimeone_via_pip_mu_nu.id)
    group_narrow_charged_DSSlnu_via_pip_via_tau: Tuple[int, ...] = (MCSigIDs.nB_to_cDone_via_pip_tau_nu.id, MCSigIDs.nB_to_cDtwostar_via_pip_tau_nu.id)
    group_wide_charged_DSSlnu_via_pip_via_tau: Tuple[int, ...] = (MCSigIDs.nB_to_cDzerostar_via_pip_tau_nu.id, MCSigIDs.nB_to_cDprimeone_via_pip_tau_nu.id)
    group_narrow_neutral_DSSlnu_via_pip_via_tau: Tuple[int, ...] = (MCSigIDs.cB_to_nDone_via_pip_tau_nu.id, MCSigIDs.cB_to_nDtwostar_via_pip_tau_nu.id)
    group_wide_neutral_DSSlnu_via_pip_via_tau: Tuple[int, ...] = (MCSigIDs.cB_to_nDzerostar_via_pip_tau_nu.id, MCSigIDs.cB_to_nDprimeone_via_pip_tau_nu.id)

    group_charged_DSSlnu_via_pip_via_e: Tuple[int, ...] = group_narrow_charged_DSSlnu_via_pip_via_e + group_wide_charged_DSSlnu_via_pip_via_e
    group_neutral_DSSlnu_via_pip_via_e: Tuple[int, ...] = group_narrow_neutral_DSSlnu_via_pip_via_e + group_wide_neutral_DSSlnu_via_pip_via_e
    group_charged_DSSlnu_via_pip_via_mu: Tuple[int, ...] = group_narrow_charged_DSSlnu_via_pip_via_mu + group_wide_charged_DSSlnu_via_pip_via_mu
    group_neutral_DSSlnu_via_pip_via_mu: Tuple[int, ...] = group_narrow_neutral_DSSlnu_via_pip_via_mu + group_wide_neutral_DSSlnu_via_pip_via_mu
    group_charged_DSSlnu_via_pip_via_tau: Tuple[int, ...] = group_narrow_charged_DSSlnu_via_pip_via_tau + group_wide_charged_DSSlnu_via_pip_via_tau
    group_neutral_DSSlnu_via_pip_via_tau: Tuple[int, ...] = group_narrow_neutral_DSSlnu_via_pip_via_tau + group_wide_neutral_DSSlnu_via_pip_via_tau

    group_narrow_DSSlnu_via_pip_via_e: Tuple[int, ...] = group_narrow_charged_DSSlnu_via_pip_via_e + group_narrow_neutral_DSSlnu_via_pip_via_e
    group_wide_DSSlnu_via_pip_via_e: Tuple[int, ...] = group_wide_charged_DSSlnu_via_pip_via_e + group_wide_neutral_DSSlnu_via_pip_via_e
    group_narrow_DSSlnu_via_pip_via_mu: Tuple[int, ...] = group_narrow_charged_DSSlnu_via_pip_via_mu + group_narrow_neutral_DSSlnu_via_pip_via_mu
    group_wide_DSSlnu_via_pip_via_mu: Tuple[int, ...] = group_wide_charged_DSSlnu_via_pip_via_mu + group_wide_neutral_DSSlnu_via_pip_via_mu
    group_narrow_DSSlnu_via_pip_via_tau: Tuple[int, ...] = group_narrow_charged_DSSlnu_via_pip_via_tau + group_narrow_neutral_DSSlnu_via_pip_via_tau
    group_wide_DSSlnu_via_pip_via_tau: Tuple[int, ...] = group_wide_charged_DSSlnu_via_pip_via_tau + group_wide_neutral_DSSlnu_via_pip_via_tau

    group_DSSlnu_via_pip_via_e: Tuple[int, ...] = group_narrow_DSSlnu_via_pip_via_e + group_wide_DSSlnu_via_pip_via_e
    group_DSSlnu_via_pip_via_mu: Tuple[int, ...] = group_narrow_DSSlnu_via_pip_via_mu + group_wide_DSSlnu_via_pip_via_mu
    group_DSSlnu_via_pip_via_tau: Tuple[int, ...] = group_narrow_DSSlnu_via_pip_via_tau + group_wide_DSSlnu_via_pip_via_tau

    group_charged_DSSlnu_via_pip_with_Done: Tuple[int, ...] = (MCSigIDs.nB_to_cDone_via_pip_e_nu.id, MCSigIDs.nB_to_cDone_via_pip_mu_nu.id, MCSigIDs.nB_to_cDone_via_pip_tau_nu.id)
    group_neutral_DSSlnu_via_pip_with_Done: Tuple[int, ...] = (MCSigIDs.cB_to_nDone_via_pip_e_nu.id, MCSigIDs.cB_to_nDone_via_pip_mu_nu.id, MCSigIDs.cB_to_nDone_via_pip_tau_nu.id)
    group_charged_DSSlnu_via_pip_with_Dtwostar: Tuple[int, ...] = (MCSigIDs.nB_to_cDtwostar_via_pip_e_nu.id, MCSigIDs.nB_to_cDtwostar_via_pip_mu_nu.id, MCSigIDs.nB_to_cDtwostar_via_pip_tau_nu.id)
    group_neutral_DSSlnu_via_pip_with_Dtwostar: Tuple[int, ...] = (MCSigIDs.cB_to_nDtwostar_via_pip_e_nu.id, MCSigIDs.cB_to_nDtwostar_via_pip_mu_nu.id, MCSigIDs.cB_to_nDtwostar_via_pip_tau_nu.id)
    group_charged_DSSlnu_via_pip_with_Dzerostar: Tuple[int, ...] = (MCSigIDs.nB_to_cDzerostar_via_pip_e_nu.id, MCSigIDs.nB_to_cDzerostar_via_pip_mu_nu.id, MCSigIDs.nB_to_cDzerostar_via_pip_tau_nu.id)
    group_neutral_DSSlnu_via_pip_with_Dzerostar: Tuple[int, ...] = (MCSigIDs.cB_to_nDzerostar_via_pip_e_nu.id, MCSigIDs.cB_to_nDzerostar_via_pip_mu_nu.id, MCSigIDs.cB_to_nDzerostar_via_pip_tau_nu.id)
    group_charged_DSSlnu_via_pip_with_Dprimeone: Tuple[int, ...] = (MCSigIDs.nB_to_cDprimeone_via_pip_e_nu.id, MCSigIDs.nB_to_cDprimeone_via_pip_mu_nu.id, MCSigIDs.nB_to_cDprimeone_via_pip_tau_nu.id)
    group_neutral_DSSlnu_via_pip_with_Dprimeone: Tuple[int, ...] = (MCSigIDs.cB_to_nDprimeone_via_pip_e_nu.id, MCSigIDs.cB_to_nDprimeone_via_pip_mu_nu.id, MCSigIDs.cB_to_nDprimeone_via_pip_tau_nu.id)

    group_narrow_charged_DSSlnu_via_pip: Tuple[int, ...] = group_charged_DSSlnu_via_pip_with_Done + group_charged_DSSlnu_via_pip_with_Dtwostar
    group_narrow_neutral_DSSlnu_via_pip: Tuple[int, ...] = group_neutral_DSSlnu_via_pip_with_Done + group_neutral_DSSlnu_via_pip_with_Dtwostar
    group_wide_charged_DSSlnu_via_pip: Tuple[int, ...] = group_charged_DSSlnu_via_pip_with_Dzerostar + group_charged_DSSlnu_via_pip_with_Dprimeone
    group_wide_neutral_DSSlnu_via_pip: Tuple[int, ...] = group_neutral_DSSlnu_via_pip_with_Dzerostar + group_neutral_DSSlnu_via_pip_with_Dprimeone

    group_charged_DSSlnu_via_pip: Tuple[int, ...] = group_narrow_charged_DSSlnu_via_pip + group_wide_charged_DSSlnu_via_pip
    group_neutral_DSSlnu_via_pip: Tuple[int, ...] = group_narrow_neutral_DSSlnu_via_pip + group_wide_neutral_DSSlnu_via_pip

    group_narrow_DSSlnu_via_pip: Tuple[int, ...] = group_DSSlnu_via_pip_with_Done + group_DSSlnu_via_pip_with_Dtwostar
    group_wide_DSSlnu_via_pip: Tuple[int, ...] = group_DSSlnu_via_pip_with_Dzerostar + group_DSSlnu_via_pip_with_Dprimeone

    group_DSSlnu_via_pip: Tuple[int, ...] = group_narrow_DSSlnu_via_pip + group_wide_DSSlnu_via_pip

    # B -> D** l nu
    group_charged_DSSlnu: Tuple[int, ...] = group_charged_DSSlnu_via_pi0 + group_charged_DSSlnu_via_pip
    group_neutral_DSSlnu: Tuple[int, ...] = group_neutral_DSSlnu_via_pi0 + group_neutral_DSSlnu_via_pip

    group_DSSlnu_with_Done: Tuple[int, ...] = group_DSSlnu_via_pi0_with_Done + group_DSSlnu_via_pip_with_Done
    group_DSSlnu_with_Dtwostar: Tuple[int, ...] = group_DSSlnu_via_pi0_with_Dtwostar + group_DSSlnu_via_pip_with_Dtwostar
    group_DSSlnu_with_Dzerostar: Tuple[int, ...] = group_DSSlnu_via_pi0_with_Dzerostar + group_DSSlnu_via_pip_with_Dzerostar
    group_DSSlnu_with_Dprimeone: Tuple[int, ...] = group_DSSlnu_via_pi0_with_Dprimeone + group_DSSlnu_via_pip_with_Dprimeone

    group_narrow_charged_DSSlnu_via_e: Tuple[int, ...] = group_narrow_charged_DSSlnu_via_pi0_via_e + group_narrow_charged_DSSlnu_via_pip_via_e
    group_wide_charged_DSSlnu_via_e: Tuple[int, ...] = group_wide_charged_DSSlnu_via_pi0_via_e + group_wide_charged_DSSlnu_via_pip_via_e
    group_narrow_neutral_DSSlnu_via_e: Tuple[int, ...] = group_narrow_neutral_DSSlnu_via_pi0_via_e + group_narrow_neutral_DSSlnu_via_pip_via_e
    group_wide_neutral_DSSlnu_via_e: Tuple[int, ...] = group_wide_neutral_DSSlnu_via_pi0_via_e + group_wide_neutral_DSSlnu_via_pip_via_e
    group_narrow_charged_DSSlnu_via_mu: Tuple[int, ...] = group_narrow_charged_DSSlnu_via_pi0_via_mu + group_narrow_charged_DSSlnu_via_pip_via_mu
    group_wide_charged_DSSlnu_via_mu: Tuple[int, ...] = group_wide_charged_DSSlnu_via_pi0_via_mu + group_wide_charged_DSSlnu_via_pip_via_mu
    group_narrow_neutral_DSSlnu_via_mu: Tuple[int, ...] = group_narrow_neutral_DSSlnu_via_pi0_via_mu + group_narrow_neutral_DSSlnu_via_pip_via_mu
    group_wide_neutral_DSSlnu_via_mu: Tuple[int, ...] = group_wide_neutral_DSSlnu_via_pi0_via_mu + group_wide_neutral_DSSlnu_via_pip_via_mu
    group_narrow_charged_DSSlnu_via_tau: Tuple[int, ...] = group_narrow_charged_DSSlnu_via_pi0_via_tau + group_narrow_charged_DSSlnu_via_pip_via_tau
    group_wide_charged_DSSlnu_via_tau: Tuple[int, ...] = group_wide_charged_DSSlnu_via_pi0_via_tau + group_wide_charged_DSSlnu_via_pip_via_tau
    group_narrow_neutral_DSSlnu_via_tau: Tuple[int, ...] = group_narrow_neutral_DSSlnu_via_pi0_via_tau + group_narrow_neutral_DSSlnu_via_pip_via_tau
    group_wide_neutral_DSSlnu_via_tau: Tuple[int, ...] = group_wide_neutral_DSSlnu_via_pi0_via_tau + group_wide_neutral_DSSlnu_via_pip_via_tau

    group_narrow_DSSlnu_via_e: Tuple[int, ...] = group_narrow_DSSlnu_via_pi0_via_e + group_narrow_DSSlnu_via_pip_via_e
    group_wide_DSSlnu_via_e: Tuple[int, ...] = group_wide_DSSlnu_via_pi0_via_e + group_wide_DSSlnu_via_pip_via_e
    group_narrow_DSSlnu_via_mu: Tuple[int, ...] = group_narrow_DSSlnu_via_pi0_via_mu + group_narrow_DSSlnu_via_pip_via_mu
    group_wide_DSSlnu_via_mu: Tuple[int, ...] = group_wide_DSSlnu_via_pi0_via_mu + group_wide_DSSlnu_via_pip_via_mu
    group_narrow_DSSlnu_via_tau: Tuple[int, ...] = group_narrow_DSSlnu_via_pi0_via_tau + group_narrow_DSSlnu_via_pip_via_tau
    group_wide_DSSlnu_via_tau: Tuple[int, ...] = group_wide_DSSlnu_via_pi0_via_tau + group_wide_DSSlnu_via_pip_via_tau

    group_charged_DSSlnu_via_e: Tuple[int, ...] = group_charged_DSSlnu_via_pi0_via_e + group_charged_DSSlnu_via_pip_via_e
    group_neutral_DSSlnu_via_e: Tuple[int, ...] = group_neutral_DSSlnu_via_pi0_via_e + group_neutral_DSSlnu_via_pip_via_e
    group_charged_DSSlnu_via_mu: Tuple[int, ...] = group_charged_DSSlnu_via_pi0_via_mu + group_charged_DSSlnu_via_pip_via_mu
    group_neutral_DSSlnu_via_mu: Tuple[int, ...] = group_neutral_DSSlnu_via_pi0_via_mu + group_neutral_DSSlnu_via_pip_via_mu
    group_charged_DSSlnu_via_tau: Tuple[int, ...] = group_charged_DSSlnu_via_pi0_via_tau + group_charged_DSSlnu_via_pip_via_tau
    group_neutral_DSSlnu_via_tau: Tuple[int, ...] = group_neutral_DSSlnu_via_pi0_via_tau + group_neutral_DSSlnu_via_pip_via_tau

    group_DSSlnu_via_e: Tuple[int, ...] = group_DSSlnu_via_pi0_via_e + group_DSSlnu_via_pip_via_e
    group_DSSlnu_via_mu: Tuple[int, ...] = group_DSSlnu_via_pi0_via_mu + group_DSSlnu_via_pip_via_mu
    group_DSSlnu_via_tau: Tuple[int, ...] = group_DSSlnu_via_pi0_via_tau + group_DSSlnu_via_pip_via_tau

    group_narrow_DSSlnu: Tuple[int, ...] = group_narrow_DSSlnu_via_pi0 + group_narrow_DSSlnu_via_pip
    group_wide_DSSlnu: Tuple[int, ...] = group_wide_DSSlnu_via_pi0 + group_wide_DSSlnu_via_pip
    group_DSSlnu: Tuple[int, ...] = group_narrow_DSSlnu + group_wide_DSSlnu

    # Hadronic Tau BKG: B -> D (tau -> hadrons) nu
    group_hadronic_tau_charged_Dtaunu_BKG: Tuple[int, ...] = (MCSigIDs.nB_to_cD_HADRONIC_tau_nu.id,)
    group_hadronic_tau_neutral_Dtaunu_BKG: Tuple[int, ...] = (MCSigIDs.cB_to_nD_HADRONIC_tau_nu.id,)

    group_hadronic_tau_Dtaunu_BKG: Tuple[int, ...] = group_hadronic_tau_charged_Dtaunu_BKG + group_hadronic_tau_neutral_Dtaunu_BKG

    # Hadronic Tau BKG: B -> D (tau -> hadrons) nu
    group_hadronic_tau_charged_DStaunu_BKG: Tuple[int, ...] = (MCSigIDs.nB_to_cDst_HADRONIC_tau_nu.id,)
    group_hadronic_tau_neutral_DStaunu_BKG: Tuple[int, ...] = (MCSigIDs.cB_to_nDst_HADRONIC_tau_nu.id,)

    group_hadronic_tau_DStaunu_BKG: Tuple[int, ...] = group_hadronic_tau_charged_DStaunu_BKG + group_hadronic_tau_neutral_DStaunu_BKG

    # Hadronic Tau BKG: B -> D(*) (tau -> hadrons) nu
    group_hadronic_tau_bkg: Tuple[int, ...] = group_hadronic_tau_Dtaunu_BKG + group_hadronic_tau_DStaunu_BKG

    # Hadronic Tau BKG: B -> D** (tau -> hadrons) nu
    group_charged_DSS_hadronic_tau_nu_with_Done_bkg: Tuple[int, ...] = (MCSigIDs.nB_to_cDone_via_pi0_HADRONIC_tau_nu.id, MCSigIDs.nB_to_cDone_via_pip_HADRONIC_tau_nu.id)
    group_neutral_DSS_hadronic_tau_nu_with_Done_bkg: Tuple[int, ...] = (MCSigIDs.cB_to_nDone_via_pi0_HADRONIC_tau_nu.id, MCSigIDs.cB_to_nDone_via_pip_HADRONIC_tau_nu.id)
    group_charged_DSS_hadronic_tau_nu_with_Dtwostar_bkg: Tuple[int, ...] = (MCSigIDs.nB_to_cDtwostar_via_pi0_HADRONIC_tau_nu.id, MCSigIDs.nB_to_cDtwostar_via_pip_HADRONIC_tau_nu.id)
    group_neutral_DSS_hadronic_tau_nu_with_Dtwostar_bkg: Tuple[int, ...] = (MCSigIDs.cB_to_nDtwostar_via_pi0_HADRONIC_tau_nu.id, MCSigIDs.cB_to_nDtwostar_via_pip_HADRONIC_tau_nu.id)
    group_charged_DSS_hadronic_tau_nu_with_Dzerostar_bkg: Tuple[int, ...] = (MCSigIDs.nB_to_cDzerostar_via_pi0_HADRONIC_tau_nu.id, MCSigIDs.nB_to_cDzerostar_via_pip_HADRONIC_tau_nu.id)
    group_neutral_DSS_hadronic_tau_nu_with_Dzerostar_bkg: Tuple[int, ...] = (MCSigIDs.cB_to_nDzerostar_via_pi0_HADRONIC_tau_nu.id, MCSigIDs.cB_to_nDzerostar_via_pip_HADRONIC_tau_nu.id)
    group_charged_DSS_hadronic_tau_nu_with_Dprimeone_bkg: Tuple[int, ...] = (MCSigIDs.nB_to_cDprimeone_via_pi0_HADRONIC_tau_nu.id, MCSigIDs.nB_to_cDprimeone_via_pip_HADRONIC_tau_nu.id)
    group_neutral_DSS_hadronic_tau_nu_with_Dprimeone_bkg: Tuple[int, ...] = (MCSigIDs.cB_to_nDprimeone_via_pi0_HADRONIC_tau_nu.id, MCSigIDs.cB_to_nDprimeone_via_pip_HADRONIC_tau_nu.id)

    group_DSS_hadronic_tau_nu_with_Done_via_pi0_bkg: Tuple[int, ...] = (MCSigIDs.nB_to_cDone_via_pi0_HADRONIC_tau_nu.id, MCSigIDs.cB_to_nDone_via_pi0_HADRONIC_tau_nu.id)
    group_DSS_hadronic_tau_nu_with_Dtwostar_via_pi0_bkg: Tuple[int, ...] = (MCSigIDs.nB_to_cDtwostar_via_pi0_HADRONIC_tau_nu.id, MCSigIDs.cB_to_nDtwostar_via_pi0_HADRONIC_tau_nu.id)
    group_DSS_hadronic_tau_nu_with_Dzerostar_via_pi0_bkg: Tuple[int, ...] = (MCSigIDs.nB_to_cDzerostar_via_pi0_HADRONIC_tau_nu.id, MCSigIDs.cB_to_nDzerostar_via_pi0_HADRONIC_tau_nu.id)
    group_DSS_hadronic_tau_nu_with_Dprimeone_via_pi0_bkg: Tuple[int, ...] = (MCSigIDs.nB_to_cDprimeone_via_pi0_HADRONIC_tau_nu.id, MCSigIDs.cB_to_nDprimeone_via_pi0_HADRONIC_tau_nu.id)

    group_DSS_hadronic_tau_nu_with_Done_via_pip_bkg: Tuple[int, ...] = (MCSigIDs.nB_to_cDone_via_pip_HADRONIC_tau_nu.id, MCSigIDs.cB_to_nDone_via_pip_HADRONIC_tau_nu.id)
    group_DSS_hadronic_tau_nu_with_Dtwostar_via_pip_bkg: Tuple[int, ...] = (MCSigIDs.nB_to_cDtwostar_via_pip_HADRONIC_tau_nu.id, MCSigIDs.cB_to_nDtwostar_via_pip_HADRONIC_tau_nu.id)
    group_DSS_hadronic_tau_nu_with_Dzerostar_via_pip_bkg: Tuple[int, ...] = (MCSigIDs.nB_to_cDzerostar_via_pip_HADRONIC_tau_nu.id, MCSigIDs.cB_to_nDzerostar_via_pip_HADRONIC_tau_nu.id)
    group_DSS_hadronic_tau_nu_with_Dprimeone_via_pip_bkg: Tuple[int, ...] = (MCSigIDs.nB_to_cDprimeone_via_pip_HADRONIC_tau_nu.id, MCSigIDs.cB_to_nDprimeone_via_pip_HADRONIC_tau_nu.id)

    group_narrow_DSS_hadronic_tau_nu_via_pi0_bkg: Tuple[int, ...] = group_DSS_hadronic_tau_nu_with_Done_via_pi0_bkg + group_DSS_hadronic_tau_nu_with_Dtwostar_via_pi0_bkg
    group_wide_DSS_hadronic_tau_nu_via_pi0_bkg: Tuple[int, ...] = group_DSS_hadronic_tau_nu_with_Dzerostar_via_pi0_bkg + group_DSS_hadronic_tau_nu_with_Dprimeone_via_pi0_bkg
    group_narrow_DSS_hadronic_tau_nu_via_pip_bkg: Tuple[int, ...] = group_DSS_hadronic_tau_nu_with_Done_via_pip_bkg + group_DSS_hadronic_tau_nu_with_Dtwostar_via_pip_bkg
    group_wide_DSS_hadronic_tau_nu_via_pip_bkg: Tuple[int, ...] = group_DSS_hadronic_tau_nu_with_Dzerostar_via_pip_bkg + group_DSS_hadronic_tau_nu_with_Dprimeone_via_pip_bkg

    group_DSS_hadronic_tau_nu_with_Done_bkg: Tuple[int, ...] = group_charged_DSS_hadronic_tau_nu_with_Done_bkg + group_neutral_DSS_hadronic_tau_nu_with_Done_bkg
    group_DSS_hadronic_tau_nu_with_Dtwostar_bkg: Tuple[int, ...] = group_charged_DSS_hadronic_tau_nu_with_Dtwostar_bkg + group_neutral_DSS_hadronic_tau_nu_with_Dtwostar_bkg
    group_DSS_hadronic_tau_nu_with_Dzerostar_bkg: Tuple[int, ...] = group_charged_DSS_hadronic_tau_nu_with_Dzerostar_bkg + group_neutral_DSS_hadronic_tau_nu_with_Dzerostar_bkg
    group_DSS_hadronic_tau_nu_with_Dprimeone_bkg: Tuple[int, ...] = group_charged_DSS_hadronic_tau_nu_with_Dprimeone_bkg + group_neutral_DSS_hadronic_tau_nu_with_Dprimeone_bkg

    group_narrow_charged_DSS_hadronic_tau_nu_bkg: Tuple[int, ...] = group_charged_DSS_hadronic_tau_nu_with_Done_bkg + group_charged_DSS_hadronic_tau_nu_with_Dtwostar_bkg
    group_narrow_neutral_DSS_hadronic_tau_nu_bkg: Tuple[int, ...] = group_neutral_DSS_hadronic_tau_nu_with_Done_bkg + group_neutral_DSS_hadronic_tau_nu_with_Dtwostar_bkg
    group_wide_charged_DSS_hadronic_tau_nu_bkg: Tuple[int, ...] = group_charged_DSS_hadronic_tau_nu_with_Dzerostar_bkg + group_charged_DSS_hadronic_tau_nu_with_Dprimeone_bkg
    group_wide_neutral_DSS_hadronic_tau_nu_bkg: Tuple[int, ...] = group_neutral_DSS_hadronic_tau_nu_with_Dzerostar_bkg + group_neutral_DSS_hadronic_tau_nu_with_Dprimeone_bkg

    group_charged_DSS_hadronic_tau_nu_bkg: Tuple[int, ...] = group_narrow_charged_DSS_hadronic_tau_nu_bkg + group_wide_charged_DSS_hadronic_tau_nu_bkg
    group_neutral_DSS_hadronic_tau_nu_bkg: Tuple[int, ...] = group_narrow_neutral_DSS_hadronic_tau_nu_bkg + group_wide_neutral_DSS_hadronic_tau_nu_bkg

    group_narrow_DSS_hadronic_tau_nu_bkg: Tuple[int, ...] = group_DSS_hadronic_tau_nu_with_Done_bkg + group_DSS_hadronic_tau_nu_with_Dtwostar_bkg
    group_wide_DSS_hadronic_tau_nu_bkg: Tuple[int, ...] = group_DSS_hadronic_tau_nu_with_Dzerostar_bkg + group_DSS_hadronic_tau_nu_with_Dprimeone_bkg

    group_DSS_hadronic_tau_bkg: Tuple[int, ...] = group_narrow_DSS_hadronic_tau_nu_bkg + group_wide_DSS_hadronic_tau_nu_bkg

    # Lepton Fake BKG  --- Hadronic Backgrounds (HBKG)
    group_lepton_fake_bkg: Tuple[int, ...] = (MCSigIDs.LeptonFake_HBKG.id, MCSigIDs.LeptonRecoFake_HBKG.id)

    # Generic Backgrounds

    # Describable Generic Backgrounds
    group_wrong_D_generic_bkg: Tuple[int, ...] = (MCSigIDs.DststMesonToThree_GBKG.id, MCSigIDs.DMesonFake_GBKG.id)
    group_unmatched_lepton_generic_bkg: Tuple[int, ...] = (MCSigIDs.LeptonWithoutBMesonRelation_GBKG.id, MCSigIDs.LeptonFromOtherBDecay_GBKG.id)
    group_other_B_to_Dxxlxx_generic_bkg: Tuple[int, ...] = (MCSigIDs.BtoDxxlxx_GBKG.id,)

    group_described_generic_bkg: Tuple[int, ...] = group_wrong_D_generic_bkg + group_unmatched_lepton_generic_bkg + group_other_B_to_Dxxlxx_generic_bkg

    # Mismatch Generic Backgrounds

    # Lepton Cross-Feed Generic Backgrounds
    group_lepton_cross_feed_direct_bkg: Tuple[int, ...] = (MCSigIDs.LeptonCrossFeed_EToMu_GBKG.id, MCSigIDs.LeptonCrossFeed_MuToE_GBKG.id)
    group_lepton_cross_feed_via_tau_bkg: Tuple[int, ...] = (MCSigIDs.LeptonCrossFeed_Tau_EToMu_GBKG.id, MCSigIDs.LeptonCrossFeed_Tau_MuToE_GBKG.id)

    group_lepton_cross_feed_EToMu_bkg: Tuple[int, ...] = (MCSigIDs.LeptonCrossFeed_EToMu_GBKG.id, MCSigIDs.LeptonCrossFeed_Tau_EToMu_GBKG.id)
    group_lepton_cross_feed_MuToE_bkg: Tuple[int, ...] = (MCSigIDs.LeptonCrossFeed_MuToE_GBKG.id, MCSigIDs.LeptonCrossFeed_Tau_MuToE_GBKG.id)

    group_lepton_cross_feed_bkg: Tuple[int, ...] = group_lepton_cross_feed_EToMu_bkg + group_lepton_cross_feed_MuToE_bkg

    # B -> D** l nu with D** daughter Mismatch Generic Backgrounds
    group_charged_DSS_daughter_mismatch_via_e_bkg: Tuple[int, ...] = (MCSigIDs.DststDaughterMismatch_nB_cDss_e_GBKG.id, MCSigIDs.DststDaughterMismatch_cB_cDss_e_GBKG.id)
    group_neutral_DSS_daughter_mismatch_via_e_bkg: Tuple[int, ...] = (MCSigIDs.DststDaughterMismatch_nB_nDss_e_GBKG.id, MCSigIDs.DststDaughterMismatch_cB_nDss_e_GBKG.id)
    group_charged_DSS_daughter_mismatch_via_mu_bkg: Tuple[int, ...] = (MCSigIDs.DststDaughterMismatch_nB_cDss_mu_GBKG.id, MCSigIDs.DststDaughterMismatch_cB_cDss_mu_GBKG.id)
    group_neutral_DSS_daughter_mismatch_via_mu_bkg: Tuple[int, ...] = (MCSigIDs.DststDaughterMismatch_nB_nDss_mu_GBKG.id, MCSigIDs.DststDaughterMismatch_cB_nDss_mu_GBKG.id)
    group_charged_DSS_daughter_mismatch_via_tau_bkg: Tuple[int, ...] = (MCSigIDs.DststDaughterMismatch_nB_cDss_tau_GBKG.id, MCSigIDs.DststDaughterMismatch_cB_cDss_tau_GBKG.id)
    group_neutral_DSS_daughter_mismatch_via_tau_bkg: Tuple[int, ...] = (MCSigIDs.DststDaughterMismatch_nB_nDss_tau_GBKG.id, MCSigIDs.DststDaughterMismatch_cB_nDss_tau_GBKG.id)

    group_DSS_daughter_mismatch_via_e_bkg: Tuple[int, ...] = group_charged_DSS_daughter_mismatch_via_e_bkg + group_neutral_DSS_daughter_mismatch_via_e_bkg
    group_DSS_daughter_mismatch_via_mu_bkg: Tuple[int, ...] = group_charged_DSS_daughter_mismatch_via_mu_bkg + group_neutral_DSS_daughter_mismatch_via_mu_bkg
    group_DSS_daughter_mismatch_via_tau_bkg: Tuple[int, ...] = group_charged_DSS_daughter_mismatch_via_tau_bkg + group_neutral_DSS_daughter_mismatch_via_tau_bkg

    group_charged_DSS_daughter_mismatch_bkg: Tuple[int, ...] = group_charged_DSS_daughter_mismatch_via_e_bkg + group_charged_DSS_daughter_mismatch_via_mu_bkg + group_charged_DSS_daughter_mismatch_via_tau_bkg
    group_neutral_DSS_daughter_mismatch_bkg: Tuple[int, ...] = group_neutral_DSS_daughter_mismatch_via_e_bkg + group_neutral_DSS_daughter_mismatch_via_mu_bkg + group_neutral_DSS_daughter_mismatch_via_tau_bkg

    group_DSS_daughter_mismatch: Tuple[int, ...] = group_charged_DSS_daughter_mismatch_bkg + group_neutral_DSS_daughter_mismatch_bkg

    # B -> pi l nu Backgrounds

    group_charged_pi_l_nu: Tuple[int, ...] = (MCSigIDs.ChargedPiLNu.id, MCSigIDs.ChargedPiTauNu.id)
    group_neutral_pi_l_nu: Tuple[int, ...] = (MCSigIDs.NeutralPiLNu.id, MCSigIDs.NeutralPiTauNu.id)

    group_charged_pi_l_nu_with_light_lepton: Tuple[int, ...] = (MCSigIDs.ChargedPiLNu.id, MCSigIDs.NeutralPiLNu.id)
    group_charged_pi_l_nu_with_tau: Tuple[int, ...] = (MCSigIDs.ChargedPiTauNu.id, MCSigIDs.NeutralPiTauNu.id)

    group_pi_l_nu: Tuple[int, ...] = group_charged_pi_l_nu + group_neutral_pi_l_nu

    group_pi_l_nu_and_described_generic_bkg: Tuple[int, ...] = group_pi_l_nu + group_described_generic_bkg

    # Other Generic B decay Backgrounds (all the events not covered by any other query)
    group_generic_bkg: Tuple[int, ...] = (MCSigIDs.GBKG.id,)

    # Continuum Backgrounds
    group_continuum_bkg: Tuple[int, ...] = (MCSigIDs.Charm_CBKG.id, MCSigIDs.UDS_CBKG.id)

    # Additional Groupings:
    group_combined_hadronic_tau_bkg: Tuple[int, ...] = group_hadronic_tau_bkg + group_DSS_hadronic_tau_bkg
    group_combined_lepton_bkg: Tuple[int, ...] = group_lepton_fake_bkg + group_lepton_cross_feed_bkg
    group_combined_generic_bkg: Tuple[int, ...] = group_generic_bkg + group_pi_l_nu_and_described_generic_bkg + group_DSS_daughter_mismatch

    # fmt: on

    @staticmethod
    def get_sig_id_groups_dict() -> Dict[str, Tuple[int, ...]]:
        _method_names: Tuple[str, ...] = (
            "get_sig_id_groups_dict",
            "get_sig_ids_of_all_groups",
        )

        return {
            attr_name: val
            for attr_name, val in MCMatchingGroups.__dict__.items()
            if not ((attr_name in _method_names) or (attr_name.endswith("__") or attr_name.startswith("_")))
        }

    @staticmethod
    def get_sig_ids_of_all_groups() -> Tuple[Tuple[int, ...], ...]:
        return tuple(sig_ids for sig_ids in MCMatchingGroups.get_sig_id_groups_dict().values())


for group_name, group_sig_ids in MCMatchingGroups.get_sig_id_groups_dict().items():
    assert all(s_id in MCMatchingGroups.all_sig_ids for s_id in group_sig_ids), (
        group_name,
        group_sig_ids,
        [s_id for s_id in group_sig_ids if s_id not in MCMatchingGroups.all_sig_ids],
    )

    assert len(set(group_sig_ids)) == len(group_sig_ids), (
        len(group_sig_ids),
        len(set(group_sig_ids)),
        set([i for i in set(group_sig_ids) if group_sig_ids.count(i) > 1]),
    )

__all_used_sig_ids: Tuple[int, ...] = tuple(set([i for g in MCMatchingGroups.get_sig_ids_of_all_groups() for i in g]))
__all_available_sig_ids: Tuple[int, ...] = MCSigIDs.get_all_sig_ids()
__used_sig_ids_check_list = [i for i in __all_available_sig_ids if i not in __all_used_sig_ids]
assert len(__used_sig_ids_check_list) == 0, (
    "Unused Sig IDs detected:",
    __used_sig_ids_check_list,
)


if __name__ == "__main__":
    __sig_id_groups_dict: Dict[str, Tuple[int, ...]] = MCMatchingGroups.get_sig_id_groups_dict()
    for k, v in __sig_id_groups_dict.items():
        print(f"{k}: {v}")

    duplicates_collection: Dict[str, List[str]] = {}
    for k, v in __sig_id_groups_dict.items():
        if any(k in dv for dv in duplicates_collection.values()):
            continue
        duplicates: List[str] = [ok for ok, ov in __sig_id_groups_dict.items() if (set(v) == set(ov) and k != ok)]
        if duplicates:
            duplicates_collection[k] = duplicates

    if duplicates_collection:
        print(
            "Duplicate keys\n\t- "
            + "\n\t- ".join(
                [
                    f"{k}: {v}"
                    + f"\n\t\t\t{k}: {__sig_id_groups_dict[k]}\n\t\t\t"
                    + "\n\t\t\t".join([f"{vk}: {__sig_id_groups_dict[vk]}" for vk in v])
                    for k, v in duplicates_collection.items()
                ]
            )
        )
        raise RuntimeError("Duplicates present! See above!")
