"""
Provides names of often used observable columns

Felix Metzner 2021
"""

__all__ = [
    "Cols",
]


class Cols:
    base_sig_id_col: str = "SIG_ID"
    extended_sig_id_col: str = "EXTENDED_SIG_ID"
    extended_sig_id__base_collection__col: str = extended_sig_id_col
    extended_sig_id__fine_collection__col: str = "EXTENDED__FINE__SIG_ID"
    extended_sig_id__simple_collection__col: str = "EXTENDED__SIMPLE__SIG_ID"
    extended_sig_id__basic_collection__col: str = "EXTENDED__BASIC__SIG_ID"
    extended_sig_id__charge_collection__col: str = "EXTENDED__CHARGE__SIG_ID"
    extended_sig_id__charge_dstar_collection__col: str = "EXTENDED__CHARGE_DS__SIG_ID"
    extended_sig_id__charge_dstar_lepton_collection__col: str = "EXTENDED__CHARGE_DS_LEPTON__SIG_ID"

    stream: str = "stream"
    data_type: str = "data_type"

    # Weight Columns
    hammered_weight_col_name: str = "__weight_final_wHAMMER_wBF_corr__"
    hammered_bgl_weight_col_name: str = "__weight_final_BGLwHAMMER_wBF_corr__"
    hammered_effort_bgl_weight_col_name: str = "__weight_final_EFFORT_BGLwHAMMER_wBF_corr__"
    hammered_cln_weight_col_name: str = "__weight_final_wOLDCLN_wBF_corr__"

    before_hammered_weight_col_name: str = "__new_weight_final_wBF_corr__"

    # fmt: off

    # Reconstruction Modes
    decay_mode_column_name: str = "daughter__bo1__cm__spextraInfo__bodecayModeID__bc__bc"
    tag_reco_mode_column_name: str = "daughter__bo0__cm__spextraInfo__bodecayModeID__bc__bc"
    u4s_reco_mode_column_name: str = "extraInfo__bodecayModeID__bc"
    primary_d_meson_reco_mode: str = "daughter__bo1__cm__spdaughter__bo0__cm__spextraInfo__bodecayModeID__bc__bc__bc"
    primary_d_meson_daughter_reco_mode: str = "daughter__bo1__cm__spdaughter__bo0__cm__spdaughter__bo0__cm__spextraInfo__bodecayModeID__bc__bc__bc__bc"

    # Event-Level MC information
    is_continuum_bkg_col: str = "isContinuumEvent"
    u4s_is_signal_col: str = "isSignalAcceptMissingNeutrino"
    b_sig_is_signal_col: str = "daughter__bo1__cm__spisSignalAcceptMissingNeutrino__bc"

    b_via_tau_index_var: str = "daughter__bo1__cm__spdaughter__bo1__cm__spmcMother__bomcMother__bomdstIndex__bc__bc__bc__bc"

    lepton_mc_mother_pdg_col: str = "daughter__bo1__cm__spdaughter__bo1__cm__spmcMother__boPDG__bc__bc__bc"

    mdst_index_via_lepton_col: str = "daughter__bo1__cm__spdaughter__bo1__cm__spmcMother__bomdstIndex__bc__bc__bc"
    mdst_index_via_tau_col: str = "daughter__bo1__cm__spdaughter__bo1__cm__spmcMother__bomcMother__bomdstIndex__bc__bc__bc__bc"

    mdst_index_via_d_meson_daughter_col: str = "daughter__bo1__cm__spdaughter__bo0__cm__spdaughter__bo0__cm__spmcMother__bomcMother__bomdstIndex__bc__bc__bc__bc__bc"
    mdst_index_via_d_meson_daughter_2_col: str = "daughter__bo1__cm__spdaughter__bo0__cm__spdaughter__bo1__cm__spmcMother__bomcMother__bomdstIndex__bc__bc__bc__bc__bc"
    mdst_index_via_d_meson_daughter_indirect_1_col: str = "daughter__bo1__cm__spdaughter__bo0__cm__spdaughter__bo0__cm__spmcMother__bomcMother__bomcMother__bomdstIndex__bc__bc__bc__bc__bc__bc"
    mdst_index_via_d_meson_daughter_indirect_2_col: str = "daughter__bo1__cm__spdaughter__bo0__cm__spdaughter__bo1__cm__spmcMother__bomcMother__bomcMother__bomdstIndex__bc__bc__bc__bc__bc__bc"
    mdst_index_via_d_meson_indirect_1_col: str = "daughter__bo1__cm__spdaughter__bo0__cm__spdaughter__bo0__cm__spmcMother__bomdstIndex__bc__bc__bc__bc"
    mdst_index_via_d_meson_indirect_2_col: str = "daughter__bo1__cm__spdaughter__bo0__cm__spdaughter__bo1__cm__spmcMother__bomdstIndex__bc__bc__bc__bc"
    # fmt: on

    # Signal Variables
    original_missing_mass: str = "m2RecoilSignalSide"
    # fmt: off
    alt_res_corr_missing_mass: str = "m2RecoilSignalSide_mbc_based_S1_CorrRes_with_AsymLaplaceSmear_RecoMode_after_2ndshift"
    # fmt: on

    missing_momentum_in_cms: str = "missingMomentumInCMSFrame"

    p_star_lepton: str = "pStarL"
    p_lepton_lab_frame: str = "daughter__bo1__cm__spdaughter__bo1__cm__spp__bc__bc"

    lepton_theta: str = "daughter__bo1__cm__spdaughter__bo1__cm__sptheta__bc__bc"

    b_sig_vertex_fit_prob: str = "daughter__bo1__cm__spchiProb__bc"

    d_inv_mass: str = "daughter__bo1__cm__spdaughter__bo0__cm__spM__bc__bc"
    d_daughter_inv_mass: str = "daughter__bo1__cm__spdaughter__bo0__cm__spdaughter__bo0__cm__spM__bc__bc__bc"

    d_mass_delta: str = "daughter__bo1__cm__spdaughter__bo0__cm__spdM__bc__bc"
    d_daughter_mass_delta: str = "daughter__bo1__cm__spdaughter__bo0__cm__spdaughter__bo0__cm__spdM__bc__bc__bc"

    # fmt: off
    diff_dstar_d_mass: str = "daughter__bo1__cm__spformula__bodaughterInvM__bo0__cm__sp2__bc__sp__mi__spdaughter__bo0__cm__spInvM__bc__bc__bc"
    # fmt: on

    # Kinematic Variables
    q_squared_reco: str = "qSquaredReco"
    w_reco: str = "wReco"

    # Tag Side Variables
    fei_probability: str = "daughter__bo0__cm__spextraInfo__boSignalProbability__bc__bc"

    b_tag_mbc: str = "daughter__bo0__cm__spMbc__bc"
    b_tag_delta_e: str = "daughter__bo0__cm__spdeltaE__bc"
    b_tag_reco_pdg: str = "daughter__bo0__cm__spPDG__bc"

    b_tag_vertex_fit_prob: str = "daughter__bo0__cm__spdaughter__bo0__cm__spchiProb__bc__bc"

    # Event Variables
    event_ne_extra: str = "ROE_neextra__boCleanROE__bc"
    event_e_extra: str = "ROE_eextra__boCleanROE__bc"

    # fmt: off
    n_clean_tracks_in_roe: str = "nCleanedTracks__bodr__sp__st__sp2__spand__spabs__bodz__bc__sp__st__sp4__bc"
    n_neutral_clusters_in_roe: str = "nCleanedECLClusters__bogoodBelleGamma__sp__eq__eq__sp1__spand__spclusterTrackMatch__sp__eq__eq__sp0__bc"
    # fmt: on

    # Continuum Variables
    is_continuum_evt: str = is_continuum_bkg_col
    is_not_continuum_evt: str = "isNotContinuumEvent"

    cs_mva_output: str = "cs_mva_output"

    cs_var_cos_tbto: str = "daughter__bo0__cm__spcosTBTO__bc"
    cs_var_cos_tbz: str = "daughter__bo0__cm__spcosTBz__bc"

    # FSR Variables
    fsr_energy_cms: str = "useCMSFrame__bofsrE__bc"
    fsr_energy_rest_frame: str = "useRestFrame__bofsrE__bc"
    fsr_energy_lab: str = "useRestFrame__bofsrE__bc"

    fsr_momentum_cms: str = "useCMSFrame__bofsrMomentum__bc"
    fsr_momentum_rest_frame: str = "useRestFrame__bofsrMomentum__bc"
    fsr_momentum_lab: str = "useRestFrame__bofsrMomentum__bc"

    # MC Truth Variables

    mc_b1_meson_pdg: str = "genUpsilon4S__bomcDaughter__bo0__cm__spPDG__bc__bc"
    mc_b2_meson_pdg: str = "genUpsilon4S__bomcDaughter__bo1__cm__spPDG__bc__bc"

    mc_b1_meson_charge: str = "genUpsilon4S__bomcDaughter__bo0__cm__spcharge__bc__bc"
    mc_b2_meson_charge: str = "genUpsilon4S__bomcDaughter__bo1__cm__spcharge__bc__bc"

    mc_b1_lepton_pdg: str = "genUpsilon4S__bomcDaughter__bo0__cm__spmcDaughter__bo1__cmPDG__bc__bc__bc"
    mc_b2_lepton_pdg: str = "genUpsilon4S__bomcDaughter__bo1__cm__spmcDaughter__bo1__cmPDG__bc__bc__bc"

    mc_b1_neutrino_pdg: str = "genUpsilon4S__bomcDaughter__bo0__cm__spmcDaughter__bo2__cmPDG__bc__bc__bc"
    mc_b2_neutrino_pdg: str = "genUpsilon4S__bomcDaughter__bo1__cm__spmcDaughter__bo2__cmPDG__bc__bc__bc"

    # fmt: off
    mc_b1_lepton_first_daughter_pdg: str = "genUpsilon4S__bomcDaughter__bo0__cm__spmcDaughter__bo1__cmmcDaughter__bo0__cmPDG__bc__bc__bc__bc"
    mc_b2_lepton_first_daughter_pdg: str = "genUpsilon4S__bomcDaughter__bo1__cm__spmcDaughter__bo1__cmmcDaughter__bo0__cmPDG__bc__bc__bc__bc"

    mc_b1_lepton_second_daughter_pdg: str = "genUpsilon4S__bomcDaughter__bo0__cm__spmcDaughter__bo1__cmmcDaughter__bo1__cmPDG__bc__bc__bc__bc"
    mc_b2_lepton_second_daughter_pdg: str = "genUpsilon4S__bomcDaughter__bo1__cm__spmcDaughter__bo1__cmmcDaughter__bo1__cmPDG__bc__bc__bc__bc"
    # fmt: on

    mc_b1_1st_degree_d_meson_pdg: str = "genUpsilon4S__bomcDaughter__bo0__cm__spmcDaughter__bo0__cmPDG__bc__bc__bc"
    mc_b2_1st_degree_d_meson_pdg: str = "genUpsilon4S__bomcDaughter__bo1__cm__spmcDaughter__bo0__cmPDG__bc__bc__bc"

    # fmt: off
    mc_b1_1st_degree_d_meson_first_daughter_pdg: str = "genUpsilon4S__bomcDaughter__bo0__cm__spmcDaughter__bo0__cmmcDaughter__bo0__cmPDG__bc__bc__bc__bc"
    mc_b2_1st_degree_d_meson_first_daughter_pdg: str = "genUpsilon4S__bomcDaughter__bo1__cm__spmcDaughter__bo0__cmmcDaughter__bo0__cmPDG__bc__bc__bc__bc"
    mc_b1_1st_degree_d_meson_second_daughter_pdg: str = "genUpsilon4S__bomcDaughter__bo0__cm__spmcDaughter__bo0__cmmcDaughter__bo1__cmPDG__bc__bc__bc__bc"
    mc_b2_1st_degree_d_meson_second_daughter_pdg: str = "genUpsilon4S__bomcDaughter__bo1__cm__spmcDaughter__bo0__cmmcDaughter__bo1__cmPDG__bc__bc__bc__bc"
    mc_b1_1st_degree_d_meson_third_daughter_pdg: str = "genUpsilon4S__bomcDaughter__bo0__cm__spmcDaughter__bo0__cmmcDaughter__bo2__cmPDG__bc__bc__bc__bc"
    mc_b2_1st_degree_d_meson_third_daughter_pdg: str = "genUpsilon4S__bomcDaughter__bo1__cm__spmcDaughter__bo0__cmmcDaughter__bo2__cmPDG__bc__bc__bc__bc"
    mc_b1_1st_degree_d_meson_fourth_daughter_pdg: str = "genUpsilon4S__bomcDaughter__bo0__cm__spmcDaughter__bo0__cmmcDaughter__bo3__cmPDG__bc__bc__bc__bc"
    mc_b2_1st_degree_d_meson_fourth_daughter_pdg: str = "genUpsilon4S__bomcDaughter__bo1__cm__spmcDaughter__bo0__cmmcDaughter__bo3__cmPDG__bc__bc__bc__bc"

    mc_b1_2nd_degree_d_meson_pdg: str = "genUpsilon4S__bomcDaughter__bo0__cm__spmcDaughter__bo0__cmmcDaughter__bo0__cmPDG__bc__bc__bc__bc"
    mc_b2_2nd_degree_d_meson_pdg: str = "genUpsilon4S__bomcDaughter__bo1__cm__spmcDaughter__bo0__cmmcDaughter__bo0__cmPDG__bc__bc__bc__bc"

    mc_b1_3rd_degree_d_meson_pdg: str = "genUpsilon4S__bomcDaughter__bo0__cm__spmcDaughter__bo0__cmmcDaughter__bo0__cmmcDaughter__bo0__cmPDG__bc__bc__bc__bc__bc"
    mc_b2_3rd_degree_d_meson_pdg: str = "genUpsilon4S__bomcDaughter__bo1__cm__spmcDaughter__bo0__cmmcDaughter__bo0__cmmcDaughter__bo0__cmPDG__bc__bc__bc__bc__bc"
    # fmt: on

    mc_b1_3rd_daughter_pdg: str = "genUpsilon4S__bomcDaughter__bo0__cm__spmcDaughter__bo2__cmPDG__bc__bc__bc"
    mc_b2_3rd_daughter_pdg: str = "genUpsilon4S__bomcDaughter__bo1__cm__spmcDaughter__bo2__cmPDG__bc__bc__bc"
    mc_b1_4th_daughter_pdg: str = "genUpsilon4S__bomcDaughter__bo0__cm__spmcDaughter__bo3__cmPDG__bc__bc__bc"
    mc_b2_4th_daughter_pdg: str = "genUpsilon4S__bomcDaughter__bo1__cm__spmcDaughter__bo3__cmPDG__bc__bc__bc"
    mc_b1_5th_daughter_pdg: str = "genUpsilon4S__bomcDaughter__bo0__cm__spmcDaughter__bo4__cmPDG__bc__bc__bc"
    mc_b2_5th_daughter_pdg: str = "genUpsilon4S__bomcDaughter__bo1__cm__spmcDaughter__bo4__cmPDG__bc__bc__bc"

    #
    # MC Matching Variables for Matching via Reconstructed Lepton
    # fmt: off
    reco_lepton_truth_matched_pdg: str = "daughter__bo1__cm__spdaughter__bo1__cm__spmatchedMC__boPDG__bc__bc__bc"

    reco_lepton_truth_matched_b_meson_pdg: str = "daughter__bo1__cm__spdaughter__bo1__cm__spmcMother__boPDG__bc__bc__bc"
    reco_lepton_truth_matched_b_meson_pdg_via_tau: str = "daughter__bo1__cm__spdaughter__bo1__cm__spmcMother__bomcMother__boPDG__bc__bc__bc__bc"

    reco_lepton_truth_matched_fourth_daughter_pdg_match: str = "daughter__bo1__cm__spdaughter__bo1__cm__spmcMother__bomcDaughter__bo3__cmPDG__bc__bc__bc__bc"
    reco_lepton_truth_matched_fourth_daughter_pdg_via_tau: str = "daughter__bo1__cm__spdaughter__bo1__cm__spmcMother__bomcMother__bomcDaughter__bo3__cmPDG__bc__bc__bc__bc__bc"

    reco_lepton_truth_matched_first_b_daugther_pdg: str = "daughter__bo1__cm__spdaughter__bo1__cm__spmcMother__bomcDaughter__bo0__cm__spPDG__bc__bc__bc__bc"
    reco_lepton_truth_matched_2nd_daugther_of_1st_daugther_pdg: str = "daughter__bo1__cm__spdaughter__bo1__cm__spmcMother__bomcDaughter__bo0__cm__spmcDaughter__bo1__cm__spPDG__bc__bc__bc__bc__bc"
    reco_lepton_truth_matched_3rd_daugther_of_1st_daugther_pdg: str = "daughter__bo1__cm__spdaughter__bo1__cm__spmcMother__bomcDaughter__bo0__cm__spmcDaughter__bo2__cm__spPDG__bc__bc__bc__bc__bc"

    # The observable below is currently not in the tuples, but used for the original SigID.
    reco_lepton_truth_matched_first_b_daughter_pdg_via_tau = "daughter__bo1__cm__spdaughter__bo1__cm__spmcMother__bomcMother__bomcDaughter__bo0__cm__spPDG__bc__bc__bc__bc__bc"
    # fmt: on

    # fmt: off
    # HAMMER Columns:

    # B meson
    mc_b1_meson_energy: str = "genUpsilon4S__bomcDaughter__bo0__cm__spE__bc__bc"
    mc_b1_meson_px: str = "genUpsilon4S__bomcDaughter__bo0__cm__sppx__bc__bc"
    mc_b1_meson_py: str = "genUpsilon4S__bomcDaughter__bo0__cm__sppy__bc__bc"
    mc_b1_meson_pz: str = "genUpsilon4S__bomcDaughter__bo0__cm__sppz__bc__bc"
    mc_b2_meson_energy: str = "genUpsilon4S__bomcDaughter__bo1__cm__spE__bc__bc"
    mc_b2_meson_px: str = "genUpsilon4S__bomcDaughter__bo1__cm__sppx__bc__bc"
    mc_b2_meson_py: str = "genUpsilon4S__bomcDaughter__bo1__cm__sppy__bc__bc"
    mc_b2_meson_pz: str = "genUpsilon4S__bomcDaughter__bo1__cm__sppz__bc__bc"

    # Lepton
    mc_b1_lepton_energy: str = "genUpsilon4S__bomcDaughter__bo0__cm__spmcDaughter__bo1__cmE__bc__bc__bc"
    mc_b1_lepton_px: str = "genUpsilon4S__bomcDaughter__bo0__cm__spmcDaughter__bo1__cmpx__bc__bc__bc"
    mc_b1_lepton_py: str = "genUpsilon4S__bomcDaughter__bo0__cm__spmcDaughter__bo1__cmpy__bc__bc__bc"
    mc_b1_lepton_pz: str = "genUpsilon4S__bomcDaughter__bo0__cm__spmcDaughter__bo1__cmpz__bc__bc__bc"
    mc_b2_lepton_energy: str = "genUpsilon4S__bomcDaughter__bo1__cm__spmcDaughter__bo1__cmE__bc__bc__bc"
    mc_b2_lepton_px: str = "genUpsilon4S__bomcDaughter__bo1__cm__spmcDaughter__bo1__cmpx__bc__bc__bc"
    mc_b2_lepton_py: str = "genUpsilon4S__bomcDaughter__bo1__cm__spmcDaughter__bo1__cmpy__bc__bc__bc"
    mc_b2_lepton_pz: str = "genUpsilon4S__bomcDaughter__bo1__cm__spmcDaughter__bo1__cmpz__bc__bc__bc"

    # Neutrino
    mc_b1_neutrino_energy: str = "genUpsilon4S__bomcDaughter__bo0__cm__spmcDaughter__bo2__cmE__bc__bc__bc"
    mc_b1_neutrino_px: str = "genUpsilon4S__bomcDaughter__bo0__cm__spmcDaughter__bo2__cmpx__bc__bc__bc"
    mc_b1_neutrino_py: str = "genUpsilon4S__bomcDaughter__bo0__cm__spmcDaughter__bo2__cmpy__bc__bc__bc"
    mc_b1_neutrino_pz: str = "genUpsilon4S__bomcDaughter__bo0__cm__spmcDaughter__bo2__cmpz__bc__bc__bc"
    mc_b2_neutrino_energy: str = "genUpsilon4S__bomcDaughter__bo1__cm__spmcDaughter__bo2__cmE__bc__bc__bc"
    mc_b2_neutrino_px: str = "genUpsilon4S__bomcDaughter__bo1__cm__spmcDaughter__bo2__cmpx__bc__bc__bc"
    mc_b2_neutrino_py: str = "genUpsilon4S__bomcDaughter__bo1__cm__spmcDaughter__bo2__cmpy__bc__bc__bc"
    mc_b2_neutrino_pz: str = "genUpsilon4S__bomcDaughter__bo1__cm__spmcDaughter__bo2__cmpz__bc__bc__bc"

    # Lepton First Daughter
    mc_b1_lepton_first_daughter_energy: str = "genUpsilon4S__bomcDaughter__bo0__cm__spmcDaughter__bo1__cm__spmcDaughter__bo0__cmE__bc__bc__bc__bc"
    mc_b1_lepton_first_daughter_px: str = "genUpsilon4S__bomcDaughter__bo0__cm__spmcDaughter__bo1__cm__spmcDaughter__bo0__cmpx__bc__bc__bc__bc"
    mc_b1_lepton_first_daughter_py: str = "genUpsilon4S__bomcDaughter__bo0__cm__spmcDaughter__bo1__cm__spmcDaughter__bo0__cmpy__bc__bc__bc__bc"
    mc_b1_lepton_first_daughter_pz: str = "genUpsilon4S__bomcDaughter__bo0__cm__spmcDaughter__bo1__cm__spmcDaughter__bo0__cmpz__bc__bc__bc__bc"
    mc_b2_lepton_first_daughter_energy: str = "genUpsilon4S__bomcDaughter__bo1__cm__spmcDaughter__bo1__cm__spmcDaughter__bo0__cmE__bc__bc__bc__bc"
    mc_b2_lepton_first_daughter_px: str = "genUpsilon4S__bomcDaughter__bo1__cm__spmcDaughter__bo1__cm__spmcDaughter__bo0__cmpx__bc__bc__bc__bc"
    mc_b2_lepton_first_daughter_py: str = "genUpsilon4S__bomcDaughter__bo1__cm__spmcDaughter__bo1__cm__spmcDaughter__bo0__cmpy__bc__bc__bc__bc"
    mc_b2_lepton_first_daughter_pz: str = "genUpsilon4S__bomcDaughter__bo1__cm__spmcDaughter__bo1__cm__spmcDaughter__bo0__cmpz__bc__bc__bc__bc"

    # Lepton Second Daughter
    mc_b1_lepton_second_daughter_energy: str = "genUpsilon4S__bomcDaughter__bo0__cm__spmcDaughter__bo1__cm__spmcDaughter__bo1__cmE__bc__bc__bc__bc"
    mc_b1_lepton_second_daughter_px: str = "genUpsilon4S__bomcDaughter__bo0__cm__spmcDaughter__bo1__cm__spmcDaughter__bo1__cmpx__bc__bc__bc__bc"
    mc_b1_lepton_second_daughter_py: str = "genUpsilon4S__bomcDaughter__bo0__cm__spmcDaughter__bo1__cm__spmcDaughter__bo1__cmpy__bc__bc__bc__bc"
    mc_b1_lepton_second_daughter_pz: str = "genUpsilon4S__bomcDaughter__bo0__cm__spmcDaughter__bo1__cm__spmcDaughter__bo1__cmpz__bc__bc__bc__bc"
    mc_b2_lepton_second_daughter_energy: str = "genUpsilon4S__bomcDaughter__bo1__cm__spmcDaughter__bo1__cm__spmcDaughter__bo1__cmE__bc__bc__bc__bc"
    mc_b2_lepton_second_daughter_px: str = "genUpsilon4S__bomcDaughter__bo1__cm__spmcDaughter__bo1__cm__spmcDaughter__bo1__cmpx__bc__bc__bc__bc"
    mc_b2_lepton_second_daughter_py: str = "genUpsilon4S__bomcDaughter__bo1__cm__spmcDaughter__bo1__cm__spmcDaughter__bo1__cmpy__bc__bc__bc__bc"
    mc_b2_lepton_second_daughter_pz: str = "genUpsilon4S__bomcDaughter__bo1__cm__spmcDaughter__bo1__cm__spmcDaughter__bo1__cmpz__bc__bc__bc__bc"

    # D Meson
    mc_b1_1st_degree_d_meson_energy: str = "genUpsilon4S__bomcDaughter__bo0__cm__spmcDaughter__bo0__cmE__bc__bc__bc"
    mc_b1_1st_degree_d_meson_px: str = "genUpsilon4S__bomcDaughter__bo0__cm__spmcDaughter__bo0__cmpx__bc__bc__bc"
    mc_b1_1st_degree_d_meson_py: str = "genUpsilon4S__bomcDaughter__bo0__cm__spmcDaughter__bo0__cmpy__bc__bc__bc"
    mc_b1_1st_degree_d_meson_pz: str = "genUpsilon4S__bomcDaughter__bo0__cm__spmcDaughter__bo0__cmpz__bc__bc__bc"
    mc_b2_1st_degree_d_meson_energy: str = "genUpsilon4S__bomcDaughter__bo1__cm__spmcDaughter__bo0__cmE__bc__bc__bc"
    mc_b2_1st_degree_d_meson_px: str = "genUpsilon4S__bomcDaughter__bo1__cm__spmcDaughter__bo0__cmpx__bc__bc__bc"
    mc_b2_1st_degree_d_meson_py: str = "genUpsilon4S__bomcDaughter__bo1__cm__spmcDaughter__bo0__cmpy__bc__bc__bc"
    mc_b2_1st_degree_d_meson_pz: str = "genUpsilon4S__bomcDaughter__bo1__cm__spmcDaughter__bo0__cmpz__bc__bc__bc"

    # D Meson First Daughter
    mc_b1_1st_degree_d_meson_first_daughter_energy: str = "genUpsilon4S__bomcDaughter__bo0__cm__spmcDaughter__bo0__cm__spmcDaughter__bo0__cmE__bc__bc__bc__bc"
    mc_b1_1st_degree_d_meson_first_daughter_px: str = "genUpsilon4S__bomcDaughter__bo0__cm__spmcDaughter__bo0__cm__spmcDaughter__bo0__cmpx__bc__bc__bc__bc"
    mc_b1_1st_degree_d_meson_first_daughter_py: str = "genUpsilon4S__bomcDaughter__bo0__cm__spmcDaughter__bo0__cm__spmcDaughter__bo0__cmpy__bc__bc__bc__bc"
    mc_b1_1st_degree_d_meson_first_daughter_pz: str = "genUpsilon4S__bomcDaughter__bo0__cm__spmcDaughter__bo0__cm__spmcDaughter__bo0__cmpz__bc__bc__bc__bc"
    mc_b2_1st_degree_d_meson_first_daughter_energy: str = "genUpsilon4S__bomcDaughter__bo1__cm__spmcDaughter__bo0__cm__spmcDaughter__bo0__cmE__bc__bc__bc__bc"
    mc_b2_1st_degree_d_meson_first_daughter_px: str = "genUpsilon4S__bomcDaughter__bo1__cm__spmcDaughter__bo0__cm__spmcDaughter__bo0__cmpx__bc__bc__bc__bc"
    mc_b2_1st_degree_d_meson_first_daughter_py: str = "genUpsilon4S__bomcDaughter__bo1__cm__spmcDaughter__bo0__cm__spmcDaughter__bo0__cmpy__bc__bc__bc__bc"
    mc_b2_1st_degree_d_meson_first_daughter_pz: str = "genUpsilon4S__bomcDaughter__bo1__cm__spmcDaughter__bo0__cm__spmcDaughter__bo0__cmpz__bc__bc__bc__bc"

    # D Meson Second Daughter
    mc_b1_1st_degree_d_meson_second_daughter_energy: str = "genUpsilon4S__bomcDaughter__bo0__cm__spmcDaughter__bo0__cm__spmcDaughter__bo1__cmE__bc__bc__bc__bc"
    mc_b1_1st_degree_d_meson_second_daughter_px: str = "genUpsilon4S__bomcDaughter__bo0__cm__spmcDaughter__bo0__cm__spmcDaughter__bo1__cmpx__bc__bc__bc__bc"
    mc_b1_1st_degree_d_meson_second_daughter_py: str = "genUpsilon4S__bomcDaughter__bo0__cm__spmcDaughter__bo0__cm__spmcDaughter__bo1__cmpy__bc__bc__bc__bc"
    mc_b1_1st_degree_d_meson_second_daughter_pz: str = "genUpsilon4S__bomcDaughter__bo0__cm__spmcDaughter__bo0__cm__spmcDaughter__bo1__cmpz__bc__bc__bc__bc"
    mc_b2_1st_degree_d_meson_second_daughter_energy: str = "genUpsilon4S__bomcDaughter__bo1__cm__spmcDaughter__bo0__cm__spmcDaughter__bo1__cmE__bc__bc__bc__bc"
    mc_b2_1st_degree_d_meson_second_daughter_px: str = "genUpsilon4S__bomcDaughter__bo1__cm__spmcDaughter__bo0__cm__spmcDaughter__bo1__cmpx__bc__bc__bc__bc"
    mc_b2_1st_degree_d_meson_second_daughter_py: str = "genUpsilon4S__bomcDaughter__bo1__cm__spmcDaughter__bo0__cm__spmcDaughter__bo1__cmpy__bc__bc__bc__bc"
    mc_b2_1st_degree_d_meson_second_daughter_pz: str = "genUpsilon4S__bomcDaughter__bo1__cm__spmcDaughter__bo0__cm__spmcDaughter__bo1__cmpz__bc__bc__bc__bc"
    # fmt: on
