"""
Reconstruction mode collections.
"""


from rdstar.utilities.logging import log_to_default

from rdstar.latex_labels.latex_process_labels import LatexDecayProcesses

from rdstar.offline_analysis.reco_modes.reco_mode_utilities import RecoModeCollection
from rdstar.offline_analysis.reco_modes.reco_mode_infos import (
    RecoModeInfos,
    TagRecoModeInfos,
    DMesonRecoModeInfos,
    U4SRecoModeInfos,
)


__all__ = [
    "BSigRecoModes",
    "AllBMesonRecoModes",
    "BtoDssLRecoModes",
    "DRecoModes",
    "BTagRecoModes",
    "U4SRecoModes",
]


logger = log_to_default(name=__name__)


class BSigRecoModes:
    main_modes: RecoModeCollection = RecoModeCollection(
        reco_mode_info_class=RecoModeInfos,
        reco_mode_infos=(
            RecoModeInfos.neutral_b_to_d_modes,
            RecoModeInfos.charged_b_to_d_modes,
            RecoModeInfos.neutral_b_to_ds_modes,
            RecoModeInfos.charged_b_to_ds_modes,
        ),
        all_reco_modes_latex_label=RecoModeInfos.get_all_reco_modes_latex_label(),
        all_reco_modes_latex_label_short=RecoModeInfos.get_all_reco_modes_latex_label_short(),
    )

    b_to_d_l_modes: RecoModeCollection = RecoModeCollection(
        reco_mode_info_class=RecoModeInfos,
        reco_mode_infos=(
            RecoModeInfos.neutral_b_to_d_modes,
            RecoModeInfos.charged_b_to_d_modes,
        ),
        all_reco_modes_latex_label=LatexDecayProcesses.all_d_meson_reco_modes_label,
        all_reco_modes_latex_label_short=LatexDecayProcesses.all_d_meson_reco_modes_label_short,
    )

    b_to_dst_l_modes: RecoModeCollection = RecoModeCollection(
        reco_mode_info_class=RecoModeInfos,
        reco_mode_infos=(
            RecoModeInfos.neutral_b_to_ds_modes,
            RecoModeInfos.charged_b_to_ds_modes,
        ),
        all_reco_modes_latex_label=LatexDecayProcesses.all_dst_meson_reco_modes_label,
        all_reco_modes_latex_label_short=LatexDecayProcesses.all_dst_meson_reco_modes_label_short,
    )

    neutral_b_to_dwwos_l_modes: RecoModeCollection = RecoModeCollection(
        reco_mode_info_class=RecoModeInfos,
        reco_mode_infos=(
            RecoModeInfos.neutral_b_to_d_modes,
            RecoModeInfos.neutral_b_to_ds_modes,
        ),
        all_reco_modes_latex_label=LatexDecayProcesses.neutral_modes_label,
        all_reco_modes_latex_label_short=LatexDecayProcesses.neutral_modes_label_short,
    )

    charged_b_to_dwwos_l_modes: RecoModeCollection = RecoModeCollection(
        reco_mode_info_class=RecoModeInfos,
        reco_mode_infos=(
            RecoModeInfos.charged_b_to_d_modes,
            RecoModeInfos.charged_b_to_ds_modes,
        ),
        all_reco_modes_latex_label=LatexDecayProcesses.charged_modes_label,
        all_reco_modes_latex_label_short=LatexDecayProcesses.charged_modes_label_short,
    )

    b_to_dwwos_l_modes_by_b_charge: RecoModeCollection = RecoModeCollection(
        reco_mode_info_class=RecoModeInfos,
        reco_mode_infos=(
            RecoModeInfos.neutral_b_to_d_wwo_star_modes,
            RecoModeInfos.charged_b_to_d_wwo_star_modes,
        ),
        all_reco_modes_latex_label=RecoModeInfos.get_all_reco_modes_latex_label(),
        all_reco_modes_latex_label_short=RecoModeInfos.get_all_reco_modes_latex_label_short(),
    )

    b_to_all_d_l_modes_by_b_charge: RecoModeCollection = RecoModeCollection(
        reco_mode_info_class=RecoModeInfos,
        reco_mode_infos=(
            RecoModeInfos.neutral_reco_modes,
            RecoModeInfos.charged_reco_modes,
        ),
        all_reco_modes_latex_label=RecoModeInfos.get_all_reco_modes_latex_label(),
        all_reco_modes_latex_label_short=RecoModeInfos.get_all_reco_modes_latex_label_short(),
    )

    detailed_main_modes: RecoModeCollection = RecoModeCollection(
        reco_mode_info_class=RecoModeInfos,
        reco_mode_infos=(
            RecoModeInfos.neutral_b_to_d_e_mode,
            RecoModeInfos.neutral_b_to_d_mu_mode,
            RecoModeInfos.charged_b_to_d_e_mode,
            RecoModeInfos.charged_b_to_d_mu_mode,
            RecoModeInfos.neutral_b_to_ds_e_mode,
            RecoModeInfos.neutral_b_to_ds_mu_mode,
            RecoModeInfos.charged_b_to_ds_e_mode,
            RecoModeInfos.charged_b_to_ds_mu_mode,
        ),
        all_reco_modes_latex_label=RecoModeInfos.get_all_reco_modes_latex_label(),
        all_reco_modes_latex_label_short=RecoModeInfos.get_all_reco_modes_latex_label_short(),
    )


class AllBMesonRecoModes:
    b_to_all_d_l_modes_by_b_charge_d_excitation_and_pi_charge: RecoModeCollection = RecoModeCollection(
        reco_mode_info_class=RecoModeInfos,
        reco_mode_infos=(
            RecoModeInfos.neutral_b_to_d_modes,
            RecoModeInfos.charged_b_to_d_modes,
            RecoModeInfos.neutral_b_to_ds_modes,
            RecoModeInfos.charged_b_to_ds_modes,
            RecoModeInfos.neutral_b_to_dss_l_via_d_pi0_modes,
            RecoModeInfos.charged_b_to_dss_l_via_d_pi0_modes,
            RecoModeInfos.neutral_b_to_dss_l_via_ds_pi0_modes,
            RecoModeInfos.charged_b_to_dss_l_via_ds_pi0_modes,
            RecoModeInfos.neutral_b_to_dss_l_via_d_pip_modes,
            RecoModeInfos.charged_b_to_dss_l_via_d_pip_modes,
            RecoModeInfos.neutral_b_to_dss_l_via_ds_pip_modes,
            RecoModeInfos.charged_b_to_dss_l_via_ds_pip_modes,
        ),
        all_reco_modes_latex_label=RecoModeInfos.get_all_reco_modes_latex_label(),
        all_reco_modes_latex_label_short=RecoModeInfos.get_all_reco_modes_latex_label_short(),
    )


class BtoDssLRecoModes:

    main_modes: RecoModeCollection = RecoModeCollection(
        reco_mode_info_class=RecoModeInfos,
        reco_mode_infos=(
            RecoModeInfos.dss_d_l_neutral_pion_modes,
            RecoModeInfos.dss_ds_l_neutral_pion_modes,
            RecoModeInfos.dss_d_l_charged_pion_modes,
            RecoModeInfos.dss_ds_l_charged_pion_modes,
        ),
        all_reco_modes_latex_label=LatexDecayProcesses.all_b_to_dss_l_modes_label,
        all_reco_modes_latex_label_short=LatexDecayProcesses.all_b_to_dss_l_modes_label_short,
    )

    main_modes_by_b_charge: RecoModeCollection = RecoModeCollection(
        reco_mode_info_class=RecoModeInfos,
        reco_mode_infos=(
            RecoModeInfos.neutral_b_to_dss_l_via_d_pi0_modes,
            RecoModeInfos.charged_b_to_dss_l_via_d_pi0_modes,
            RecoModeInfos.neutral_b_to_dss_l_via_ds_pi0_modes,
            RecoModeInfos.charged_b_to_dss_l_via_ds_pi0_modes,
            RecoModeInfos.neutral_b_to_dss_l_via_d_pip_modes,
            RecoModeInfos.charged_b_to_dss_l_via_d_pip_modes,
            RecoModeInfos.neutral_b_to_dss_l_via_ds_pip_modes,
            RecoModeInfos.charged_b_to_dss_l_via_ds_pip_modes,
        ),
        all_reco_modes_latex_label=LatexDecayProcesses.all_b_to_dss_l_modes_label,
        all_reco_modes_latex_label_short=LatexDecayProcesses.all_b_to_dss_l_modes_label_short,
    )

    neutral_b_to_dss_l_modes: RecoModeCollection = RecoModeCollection(
        reco_mode_info_class=RecoModeInfos,
        reco_mode_infos=(
            RecoModeInfos.neutral_b_to_dss_l_via_d_modes,
            RecoModeInfos.neutral_b_to_dss_l_via_ds_modes,
        ),
        all_reco_modes_latex_label=LatexDecayProcesses.neutral_b_to_dss_l_modes_label,
        all_reco_modes_latex_label_short=LatexDecayProcesses.neutral_b_to_dss_l_modes_label_short,
    )

    charged_b_to_dss_l_modes: RecoModeCollection = RecoModeCollection(
        reco_mode_info_class=RecoModeInfos,
        reco_mode_infos=(
            RecoModeInfos.charged_b_to_dss_l_via_d_modes,
            RecoModeInfos.charged_b_to_dss_l_via_ds_modes,
        ),
        all_reco_modes_latex_label=LatexDecayProcesses.charged_b_to_dss_l_modes_label,
        all_reco_modes_latex_label_short=LatexDecayProcesses.charged_b_to_dss_l_modes_label_short,
    )

    b_to_dss_l_modes: RecoModeCollection = RecoModeCollection(
        reco_mode_info_class=RecoModeInfos,
        reco_mode_infos=(
            RecoModeInfos.neutral_b_to_dss_l_via_d_modes,
            RecoModeInfos.charged_b_to_dss_l_via_d_modes,
            RecoModeInfos.neutral_b_to_dss_l_via_ds_modes,
            RecoModeInfos.charged_b_to_dss_l_via_ds_modes,
        ),
        all_reco_modes_latex_label=LatexDecayProcesses.all_b_to_dss_l_modes_label,
        all_reco_modes_latex_label_short=LatexDecayProcesses.all_b_to_dss_l_modes_label_short,
    )

    neutral_b_to_dss_l_with_neutral_pi_modes: RecoModeCollection = RecoModeCollection(
        reco_mode_info_class=RecoModeInfos,
        reco_mode_infos=(
            RecoModeInfos.neutral_b_to_dss_l_via_d_pi0_modes,
            RecoModeInfos.neutral_b_to_dss_l_via_ds_pi0_modes,
        ),
        all_reco_modes_latex_label=LatexDecayProcesses.all_reco_b0_to_d_wwos_pi0_l_label,
        all_reco_modes_latex_label_short=LatexDecayProcesses.all_reco_b0_to_d_wwos_pi0_l_label_short,
    )

    charged_b_to_dss_l_with_neutral_pi_modes: RecoModeCollection = RecoModeCollection(
        reco_mode_info_class=RecoModeInfos,
        reco_mode_infos=(
            RecoModeInfos.charged_b_to_dss_l_via_d_pi0_modes,
            RecoModeInfos.charged_b_to_dss_l_via_ds_pi0_modes,
        ),
        all_reco_modes_latex_label=LatexDecayProcesses.all_reco_bpm_to_d_wwos_pi0_l_label,
        all_reco_modes_latex_label_short=LatexDecayProcesses.all_reco_bpm_to_d_wwos_pi0_l_label_short,
    )

    b_to_dss_l_with_neutral_pi_modes: RecoModeCollection = RecoModeCollection(
        reco_mode_info_class=RecoModeInfos,
        reco_mode_infos=(
            RecoModeInfos.neutral_b_to_dss_l_via_d_pi0_modes,
            RecoModeInfos.charged_b_to_dss_l_via_d_pi0_modes,
            RecoModeInfos.neutral_b_to_dss_l_via_ds_pi0_modes,
            RecoModeInfos.charged_b_to_dss_l_via_ds_pi0_modes,
        ),
        all_reco_modes_latex_label=LatexDecayProcesses.all_reco_b_to_d_wwos_pi0_l_label,
        all_reco_modes_latex_label_short=LatexDecayProcesses.all_reco_b_to_d_wwos_pi0_l_label_short,
    )

    neutral_b_to_dss_l_with_charged_pi_modes: RecoModeCollection = RecoModeCollection(
        reco_mode_info_class=RecoModeInfos,
        reco_mode_infos=(
            RecoModeInfos.neutral_b_to_dss_l_via_d_pip_modes,
            RecoModeInfos.neutral_b_to_dss_l_via_ds_pip_modes,
        ),
        all_reco_modes_latex_label=LatexDecayProcesses.all_reco_b0_to_d_wwos_pip_l_label,
        all_reco_modes_latex_label_short=LatexDecayProcesses.all_reco_b0_to_d_wwos_pip_l_label_short,
    )

    charged_b_to_dss_l_with_charged_pi_modes: RecoModeCollection = RecoModeCollection(
        reco_mode_info_class=RecoModeInfos,
        reco_mode_infos=(
            RecoModeInfos.charged_b_to_dss_l_via_d_pip_modes,
            RecoModeInfos.charged_b_to_dss_l_via_ds_pip_modes,
        ),
        all_reco_modes_latex_label=LatexDecayProcesses.all_reco_bpm_to_d_wwos_pip_l_label,
        all_reco_modes_latex_label_short=LatexDecayProcesses.all_reco_bpm_to_d_wwos_pip_l_label_short,
    )

    b_to_dss_l_with_charged_pi_modes: RecoModeCollection = RecoModeCollection(
        reco_mode_info_class=RecoModeInfos,
        reco_mode_infos=(
            RecoModeInfos.neutral_b_to_dss_l_via_d_pip_modes,
            RecoModeInfos.charged_b_to_dss_l_via_d_pip_modes,
            RecoModeInfos.neutral_b_to_dss_l_via_ds_pip_modes,
            RecoModeInfos.charged_b_to_dss_l_via_ds_pip_modes,
        ),
        all_reco_modes_latex_label=LatexDecayProcesses.all_reco_b_to_d_wwos_pip_l_label,
        all_reco_modes_latex_label_short=LatexDecayProcesses.all_reco_b_to_d_wwos_pip_l_label_short,
    )

    b_to_dss_l_modes_by_d_charge: RecoModeCollection = RecoModeCollection(
        reco_mode_info_class=RecoModeInfos,
        reco_mode_infos=(
            RecoModeInfos.dss_via_charged_dwwos_l_pion_modes,
            RecoModeInfos.dss_via_neutral_dwwos_l_pion_modes,
        ),
        all_reco_modes_latex_label=LatexDecayProcesses.all_b_to_dss_l_modes_label,
        all_reco_modes_latex_label_short=LatexDecayProcesses.all_b_to_dss_l_modes_label_short,
    )

    b_to_dss_l_modes_by_pi_charge: RecoModeCollection = RecoModeCollection(
        reco_mode_info_class=RecoModeInfos,
        reco_mode_infos=(
            RecoModeInfos.d_ss_neutral_pion_reco_modes,
            RecoModeInfos.d_ss_charged_pion_reco_modes,
        ),
        all_reco_modes_latex_label=LatexDecayProcesses.all_b_to_dss_l_modes_label,
        all_reco_modes_latex_label_short=LatexDecayProcesses.all_b_to_dss_l_modes_label_short,
    )

    b_to_dss_l_modes_by_b_charge_and_pi_charge: RecoModeCollection = RecoModeCollection(
        reco_mode_info_class=RecoModeInfos,
        reco_mode_infos=(
            RecoModeInfos.neutral_b_to_dss_l_via_pi0_modes,
            RecoModeInfos.neutral_b_to_dss_l_via_pip_modes,
            RecoModeInfos.charged_b_to_dss_l_via_pi0_modes,
            RecoModeInfos.charged_b_to_dss_l_via_pip_modes,
        ),
        all_reco_modes_latex_label=LatexDecayProcesses.all_b_to_dss_l_modes_label,
        all_reco_modes_latex_label_short=LatexDecayProcesses.all_b_to_dss_l_modes_label_short,
    )

    b_to_dss_l_modes_by_b_charge_and_d_excitation: RecoModeCollection = RecoModeCollection(
        reco_mode_info_class=RecoModeInfos,
        reco_mode_infos=(
            RecoModeInfos.dss_neutral_b_to_dss_pi_l_via_d_modes,
            RecoModeInfos.dss_neutral_b_to_dss_pi_l_via_ds_modes,
            RecoModeInfos.dss_charged_b_to_dss_pi_l_via_d_modes,
            RecoModeInfos.dss_charged_b_to_dss_pi_l_via_ds_modes,
        ),
        all_reco_modes_latex_label=LatexDecayProcesses.all_b_to_dss_l_modes_label,
        all_reco_modes_latex_label_short=LatexDecayProcesses.all_b_to_dss_l_modes_label_short,
    )

    b_to_dss_l_modes_in_various_combinations: RecoModeCollection = RecoModeCollection(
        reco_mode_info_class=RecoModeInfos,
        reco_mode_infos=(
            # BtoDssLRecoModes.main_modes
            RecoModeInfos.dss_d_l_neutral_pion_modes,
            RecoModeInfos.dss_ds_l_neutral_pion_modes,
            RecoModeInfos.dss_d_l_charged_pion_modes,
            RecoModeInfos.dss_ds_l_charged_pion_modes,
            # BtoDssLRecoModes.main_modes_by_b_charge
            RecoModeInfos.neutral_b_to_dss_l_via_d_pi0_modes,
            RecoModeInfos.charged_b_to_dss_l_via_d_pi0_modes,
            RecoModeInfos.neutral_b_to_dss_l_via_ds_pi0_modes,
            RecoModeInfos.charged_b_to_dss_l_via_ds_pi0_modes,
            RecoModeInfos.neutral_b_to_dss_l_via_d_pip_modes,
            RecoModeInfos.charged_b_to_dss_l_via_d_pip_modes,
            RecoModeInfos.neutral_b_to_dss_l_via_ds_pip_modes,
            RecoModeInfos.charged_b_to_dss_l_via_ds_pip_modes,
            # BtoDssLRecoModes.b_to_dss_l_modes_by_pi_charge
            RecoModeInfos.d_ss_neutral_pion_reco_modes,
            RecoModeInfos.d_ss_charged_pion_reco_modes,
            # BtoDssLRecoModes.b_to_dss_l_modes_by_b_charge_and_pi_charge
            RecoModeInfos.neutral_b_to_dss_l_via_pi0_modes,
            RecoModeInfos.neutral_b_to_dss_l_via_pip_modes,
            RecoModeInfos.charged_b_to_dss_l_via_pi0_modes,
            RecoModeInfos.charged_b_to_dss_l_via_pip_modes,
            # BtoDssLRecoModes.b_to_dss_l_modes_by_b_charge_and_d_excitation
            RecoModeInfos.dss_neutral_b_to_dss_pi_l_via_d_modes,
            RecoModeInfos.dss_neutral_b_to_dss_pi_l_via_ds_modes,
            RecoModeInfos.dss_charged_b_to_dss_pi_l_via_d_modes,
            RecoModeInfos.dss_charged_b_to_dss_pi_l_via_ds_modes,
        ),
        all_reco_modes_latex_label=LatexDecayProcesses.all_b_to_dss_l_modes_label,
        all_reco_modes_latex_label_short=LatexDecayProcesses.all_b_to_dss_l_modes_label_short,
        main_reco_mode_infos=main_modes_by_b_charge.reco_mode_infos,
        allow_duplicates=True,
    )


class DRecoModes:
    main_modes: RecoModeCollection = RecoModeCollection(
        reco_mode_info_class=DMesonRecoModeInfos,
        reco_mode_infos=(
            DMesonRecoModeInfos.dp_modes,
            DMesonRecoModeInfos.d0_modes,
            DMesonRecoModeInfos.dstp_modes,
            DMesonRecoModeInfos.dst0_modes,
        ),
        all_reco_modes_latex_label=DMesonRecoModeInfos.get_all_reco_modes_latex_label(),
        all_reco_modes_latex_label_short=DMesonRecoModeInfos.get_all_reco_modes_latex_label_short(),
    )


class BTagRecoModes:
    main_modes: RecoModeCollection = RecoModeCollection(
        reco_mode_info_class=TagRecoModeInfos,
        reco_mode_infos=(
            TagRecoModeInfos.neutral_tag_reco_modes,
            TagRecoModeInfos.charged_tag_reco_modes,
        ),
        all_reco_modes_latex_label=TagRecoModeInfos.get_all_reco_modes_latex_label(),
        all_reco_modes_latex_label_short=TagRecoModeInfos.get_all_reco_modes_latex_label_short(),
    )

    b_tag_by_groups: RecoModeCollection = RecoModeCollection(
        reco_mode_info_class=TagRecoModeInfos,
        reco_mode_infos=(
            TagRecoModeInfos.b_to_d_1cpi_tag_modes,
            TagRecoModeInfos.b_to_d_mcpi_tag_modes,
            TagRecoModeInfos.b_to_ocd_pipi_tag_modes,
            TagRecoModeInfos.b_to_ds_1cpi_tag_modes,
            TagRecoModeInfos.b_to_ds_mcpi_tag_modes,
            TagRecoModeInfos.b_to_j_psi_tag_modes,
            TagRecoModeInfos.b_to__all_dd__tag_modes,
        ),
        all_reco_modes_latex_label=TagRecoModeInfos.get_all_reco_modes_latex_label(),
        all_reco_modes_latex_label_short=TagRecoModeInfos.get_all_reco_modes_latex_label_short(),
    )

    b_tag_by_groups_and_by_b_charge: RecoModeCollection = RecoModeCollection(
        reco_mode_info_class=TagRecoModeInfos,
        reco_mode_infos=(
            TagRecoModeInfos.charged_b_to_d_1cpi_tag_modes,
            TagRecoModeInfos.neutral_b_to_d_1cpi_tag_modes,
            TagRecoModeInfos.charged_b_to_d_mcpi_tag_modes,
            TagRecoModeInfos.neutral_b_to_d_mcpi_tag_modes,
            TagRecoModeInfos.charged_b_to_ocd_pipi_tag_modes,
            TagRecoModeInfos.neutral_b_to_ocd_pipi_tag_modes,
            TagRecoModeInfos.charged_b_to_ds_1cpi_tag_modes,
            TagRecoModeInfos.neutral_b_to_ds_1cpi_tag_modes,
            TagRecoModeInfos.charged_b_to_ds_mcpi_tag_modes,
            TagRecoModeInfos.neutral_b_to_ds_mcpi_tag_modes,
            TagRecoModeInfos.charged_b_to_j_psi_tag_modes,
            TagRecoModeInfos.neutral_b_to_j_psi_tag_modes,
            TagRecoModeInfos.charged_b_to__all_dd__tag_modes,
            TagRecoModeInfos.neutral_b_to__all_dd__tag_modes,
        ),
        all_reco_modes_latex_label=TagRecoModeInfos.get_all_reco_modes_latex_label(),
        all_reco_modes_latex_label_short=TagRecoModeInfos.get_all_reco_modes_latex_label_short(),
    )


class U4SRecoModes:
    main_modes: RecoModeCollection = RecoModeCollection(
        reco_mode_info_class=U4SRecoModeInfos,
        reco_mode_infos=(
            U4SRecoModeInfos.u4s_from_charged_reco_modes,
            U4SRecoModeInfos.u4s_from_all_neutral_reco_modes,
        ),
        all_reco_modes_latex_label=U4SRecoModeInfos.get_all_reco_modes_latex_label(),
        all_reco_modes_latex_label_short=U4SRecoModeInfos.get_all_reco_modes_latex_label_short(),
    )


if __name__ == "__main__":
    print(len(BSigRecoModes.main_modes))
    print([i for i in BSigRecoModes.main_modes])
    print([i.ids for i in BSigRecoModes.main_modes.values()])

    print(AllBMesonRecoModes.b_to_all_d_l_modes_by_b_charge_d_excitation_and_pi_charge.all_reco_mode_ids)
