"""
Additional special selection methods

Felix Metzner 2023
"""

import pandas as pd

from typing import Tuple

from rdstar.offline_analysis.observables.common_observables import Cols


__all__ = [
    "apply_d_meson_reco_channel_reduction_selection",
]


primary_d_reco_mode_col_name = Cols.primary_d_meson_reco_mode
secondary_d_reco_mode_col_name = Cols.primary_d_meson_daughter_reco_mode


main_d_reco_mode_ids: Tuple[int, ...] = (
    31,  # "D+:Kpipi": 9.46 %
    34,  # "D+:KSpi": 1.53 %
    35,  # "D+:KSpipi": 7.24 %
    36,  # "D+:KSpipipi": 3.05 %
    #
    41,  # "D0:Kpi": 3.93 %
    42,  # "D0:Kpipi": 14.3 %
    43,  # "D0:Kpipipi": 8.07 %
    45,  # "D0:KSpi": 1.20 %
    46,  # "D0:KSpipi": 2.85 %
)

new_d_reco_mode_ids: Tuple[int, ...] = (
    32,  # "D+:Kpipipi": 6.14 %
    33,  # "D+:Kpipipipi": 5.8 E-3
    37,  # "D+:KSK": 2.95 E-3
    38,  # "D+:KKpi": 9.96 E-3
    #
    44,  # "D0:Kpipipipi": 4.2 %
    47,  # "D0:KSpipipi": 5.2 %
    48,  # "D0:KK": 4.01 E-3
)

assert len(set(main_d_reco_mode_ids).intersection(set(new_d_reco_mode_ids))) == 0, (
    main_d_reco_mode_ids,
    new_d_reco_mode_ids,
)


def apply_d_meson_reco_channel_reduction_selection(df: pd.DataFrame) -> pd.DataFrame:
    assert primary_d_reco_mode_col_name in df.columns, primary_d_reco_mode_col_name
    assert secondary_d_reco_mode_col_name in df.columns, secondary_d_reco_mode_col_name

    is_main_primary_d: pd.Series = df[primary_d_reco_mode_col_name].isin(main_d_reco_mode_ids)
    is_main_secondary_d: pd.Series = df[secondary_d_reco_mode_col_name].isin(main_d_reco_mode_ids)

    is_main_d: pd.Series = is_main_primary_d | is_main_secondary_d

    return df[is_main_d].reset_index(drop=True)
